@@ -3,14 +3,16 @@
  * library.c
  *
  *   Canon Camera library for the gphoto project,
- *   (c) 1999 Wolfgang G. Reissnegger
+ *    1999 Wolfgang G. Reissnegger
  *   Developed for the Canon PowerShot A50
  *   Additions for PowerShot A5 by Ole W. Saastad
- *   (c) 2000: Other additions  by Edouard Lafargue, Philippe Marzouk
+ *    2000: Other additions  by Edouard Lafargue, Philippe Marzouk
  *
  * This file contains all the "glue code" required to use the canon
  * driver with libgphoto2.
  *
+ * $Id: library.c,v 1.83 2004/11/17 20:28:34 swestin Exp $
+ *
  ****************************************************************************/
 
 
@@ -31,6 +33,7 @@
 #include <sys/stat.h>
 #include <termios.h>
 #include <time.h>
+#include <ctype.h>
 
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -64,8 +67,10 @@
 #include "usb.h"
 
 #ifndef HAVE_SNPRINTF
-#warning You do not seem to have a snprintf() function. Using sprintf instead.
-#warning Note that this leads to SECURITY RISKS!
+# ifdef __GCC__
+#  warning You do not seem to have a snprintf() function. Using sprintf instead.
+#  warning Note that this leads to SECURITY RISKS!
+# endif
 #define snprintf(buf,size,format,arg) sprintf(buf,format,arg)
 #endif
 
@@ -83,19 +88,22 @@ extern long int timezone;
 # define FALSE (0!=0)
 #endif
 
-#ifdef CANON_EXPERIMENTAL_CAPTURE
-#define CAPTURE_BOOL TRUE
-#else
-#define CAPTURE_BOOL FALSE
-#endif
-
 #ifdef CANON_EXPERIMENTAL_UPLOAD
 #define UPLOAD_BOOL TRUE
 #else
 #define UPLOAD_BOOL FALSE
 #endif
 
-
+/**
+ * camera_id:
+ * @id: string buffer to receive text identifying camera type
+ *
+ * Standard gphoto2 camlib interface
+ *
+ * Returns:
+ *  string is copied into @id.
+ *
+ */
 int
 camera_id (CameraText *id)
 {
@@ -106,20 +114,42 @@ camera_id (CameraText *id)
 	return GP_OK;
 }
 
+/**
+ * camera_manual
+ * @camera: Camera for which to get manual text (unused)
+ * @manual: Buffer into which to copy manual text
+ * @context: unused
+ *
+ * Returns the "manual" text for cameras supported by this driver.
+ *
+ * Returns: manual text is copied into @manual->text.
+ *
+ */
 static int
 camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 {
 	GP_DEBUG ("camera_manual()");
 
 	strcpy (manual->text,
-		_("For the A50, 115200 may not be faster than 57600\n"
-		  "Folders are NOT supported\n"
-		  "if you experience a lot of transmissions errors,"
-		  " try to have you computer as idle as possible (ie: no disk activity)"));
+		_("For the A50, using 115200 may effectively be slower than using 57600\n"
+		  "If you experience a lot of serial transmission errors,"
+		  " try to have your computer as idle as possible\n"
+		  " (i.e. no disk activity)\n"
+			));
 
 	return GP_OK;
 }
 
+/**
+ * camera_abilities:
+ * @list: list of abilities
+ *
+ * Returns a list of what any of the cameras supported by this driver
+ *   can do. Each entry of the list will represent one camera model.
+ *
+ * Returns: list of abilities in @list
+ *
+ */
 int
 camera_abilities (CameraAbilitiesList *list)
 {
@@ -131,7 +161,7 @@ camera_abilities (CameraAbilitiesList *list)
 	for (i = 0; models[i].id_str; i++) {
 		memset (&a, 0, sizeof (a));
 
-		if ((CAPTURE_BOOL || UPLOAD_BOOL) && 
+		if ((UPLOAD_BOOL || (models[i].usb_capture_support == CAP_EXP)) && 
 		    (models[i].usb_vendor && models[i].usb_product)) {
 			a.status = GP_DRIVER_STATUS_EXPERIMENTAL;
 		} else {
@@ -145,7 +175,7 @@ camera_abilities (CameraAbilitiesList *list)
 			a.usb_vendor = models[i].usb_vendor;
 			a.usb_product = models[i].usb_product;
 		}
-		if (models[i].serial_support) {
+		if (models[i].serial_id_string != NULL) {
 			a.port |= GP_PORT_SERIAL;
 			a.speed[0] = 9600;
 			a.speed[1] = 19200;
@@ -155,25 +185,34 @@ camera_abilities (CameraAbilitiesList *list)
 			a.speed[5] = 0;
 		}
 		a.operations = GP_OPERATION_CONFIG;
-#ifdef CANON_EXPERIMENTAL_CAPTURE
-		a.operations |= GP_OPERATION_CAPTURE_IMAGE |
-			GP_OPERATION_CAPTURE_PREVIEW;
-#endif
+
+		if (models[i].usb_capture_support != CAP_NON) {
+			a.operations |= GP_OPERATION_CAPTURE_IMAGE;
+			a.operations |= GP_OPERATION_CAPTURE_PREVIEW;
+		}
+
 		a.folder_operations =
-#ifdef CANON_EXPERIMENTAL_UPLOAD
-			GP_FOLDER_OPERATION_PUT_FILE 
-#else
-			(models[i].serial_support?GP_FOLDER_OPERATION_PUT_FILE:0)
-#endif
-			| GP_FOLDER_OPERATION_MAKE_DIR |
+			GP_FOLDER_OPERATION_MAKE_DIR |
 			GP_FOLDER_OPERATION_REMOVE_DIR;
-		a.file_operations = GP_FILE_OPERATION_DELETE | GP_FILE_OPERATION_PREVIEW;
+
+		if (UPLOAD_BOOL || (models[i].serial_id_string != NULL)) {
+			a.folder_operations |= GP_FOLDER_OPERATION_PUT_FILE;
+		}
+
+		a.file_operations = GP_FILE_OPERATION_DELETE | GP_FILE_OPERATION_PREVIEW | GP_FILE_OPERATION_EXIF;
 		gp_abilities_list_append (list, a);
 	}
 
 	return GP_OK;
 }
 
+/**
+ * clear_readiness
+ * @camera: the camera to affect
+ *
+ * Clears the cached flag indicating that the camera is ready.
+ *
+ */
 void
 clear_readiness (Camera *camera)
 {
@@ -182,6 +221,17 @@ clear_readiness (Camera *camera)
 	camera->pl->cached_ready = 0;
 }
 
+/**
+ * check_readiness
+ * @camera: the camera to affect
+ * @context: context for error reporting
+ *
+ * Checks the readiness of the camera. If the cached "ready" flag isn't
+ *  set, checks using canon_int_ready().
+ *
+ * Returns: 1 if ready, 0 if not.
+ *
+ */
 static int
 check_readiness (Camera *camera, GPContext *context)
 {
@@ -199,6 +249,14 @@ check_readiness (Camera *camera, GPContext *context)
 	return 0;
 }
 
+/**
+ * canon_int_switch_camera_off
+ * @camera: the camera to affect
+ * @context: context for error reporting
+ *
+ * Switches a serial camera off. Does nothing to a USB camera.
+ *
+ */
 static void
 canon_int_switch_camera_off (Camera *camera, GPContext *context)
 {
@@ -216,11 +274,24 @@ canon_int_switch_camera_off (Camera *camera, GPContext *context)
 	clear_readiness (camera);
 }
 
+/**
+ * camera_exit
+ * @camera: the camera to affect
+ * @context: context for error reporting
+ *
+ * Ends our use of the camera. For USB cameras, unlock keys. For
+ *  serial cameras, switch the camera off and unlock the port.
+ *
+ * Returns: GP_OK
+ *
+ */
 static int
 camera_exit (Camera *camera, GPContext *context)
 {
 	if (camera->port->type == GP_PORT_USB) {
-		canon_usb_unlock_keys (camera);
+		/* EOS-class cameras lock keys only for remote
+		   capture, and unlock immediately. */
+		if ( !IS_EOS(camera->pl->md->model) ) canon_usb_unlock_keys (camera, context);
 	}
 
 	if (camera->pl) {
@@ -232,46 +303,82 @@ camera_exit (Camera *camera, GPContext *context)
 	return GP_OK;
 }
 
-#ifdef CANON_EXPERIMENTAL_CAPTURE
-
+/**
+ * camera_capture_preview
+ * @camera: the camera to affect
+ * @context: context for error reporting
+ *
+ * Triggers the camera to capture a new image. Discards the image and
+ * downloads its thumbnail to the host computer without storing it on
+ * the camera.
+ *
+ * Returns: GP_OK
+ *
+ */
 static int
 camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 {
-	GP_DEBUG ("canon_capture_preview() called, currently not implemented");
+	int size;
+	unsigned char *data;
+
+	GP_DEBUG ("canon_capture_preview() called");
+
+	if (canon_int_capture_preview (camera, &data, &size, context) != GP_OK) {
+		gp_context_error (context, _("Error capturing image"));
+		return GP_ERROR;
+	}
+	gp_file_set_data_and_size ( file, data, size );
+	gp_file_set_mime_type (file, GP_MIME_JPEG);	/* always */
+	/* Add an arbitrary file name so caller won't crash */
+	gp_file_set_name (file, "canon_preview.jpg");
 
 	return GP_OK;
 }
 
+/**
+ * camera_capture
+ * @camera: the camera to affect
+ * @type: type of capture (only GP_CAPTURE_IMAGE supported)
+ * @path: path on camera to use
+ * @context: context for error reporting
+ *
+ * Captures a single image from the camera.
+ *
+ * Returns: gphoto2 error code
+ *
+ */
 static int
 camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
 		GPContext *context)
 {
 	GP_DEBUG ("canon_capture() called");
 
-	GP_DEBUG ("CANON REMOTE CAPTURE IS EXPERIMENTAL CODE, WORK IN PROGRESS "
-		  "AND MAYBE NOT WORKING AT ALL. NOT FOR THE FAINT OF HEART.");
-
-	/* As this is experimental code anyway, we don't test for the
-	 * camera here. And if we wanted to, there would be another
-	 * place better suited (in camera_init()). */
-
 	if (type != GP_CAPTURE_IMAGE) {
 		return GP_ERROR_NOT_SUPPORTED;
 	}
 
-	camera->pl->capturing = TRUE;
 	if (canon_int_capture_image (camera, path, context) != GP_OK) {
-		camera->pl->capturing = FALSE;
 		gp_context_error (context, _("Error capturing image"));
 		return GP_ERROR;
 	}
 
-	camera->pl->capturing = FALSE;
 	return GP_OK;
 }
 
-#endif /* CANON_EXPERIMENTAL_CAPTURE */
-
+/**
+ * canon_get_batt_status
+ * @camera: the camera to affect
+ * @pwr_status: to receive power status from camera
+ * @pwr_source: to receive power source from camera
+ * @context: context for error reporting
+ *
+ * Gets the power status for the camera.
+ *
+ * Returns: -1 if camera isn't ready; status from canon_int_get_battery() otherwise.
+ *   @pwr_status will contain @CAMERA_POWER_OK if power is OK (i.e. battery isn't low)
+ *   @pwr_source will have bit %CAMERA_MASK_BATTERY set if battery power is used
+ *
+ */
 static int
 canon_get_batt_status (Camera *camera, int *pwr_status, int *pwr_source, GPContext *context)
 {
@@ -283,6 +390,16 @@ canon_get_batt_status (Camera *camera, int *pwr_status, int *pwr_source, GPConte
 	return canon_int_get_battery (camera, pwr_status, pwr_source, context);
 }
 
+/**
+ * update_disk_cache
+ * @camera: the camera to work on
+ * @context: the context to print on error
+ *
+ * Updates the disk cache for this camera.
+ *
+ * Returns: 1 on success, 0 on failure
+ *
+ */
 static int
 update_disk_cache (Camera *camera, GPContext *context)
 {
@@ -355,37 +472,50 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	Camera *camera = user_data;
 	unsigned char *data = NULL, *thumbdata = NULL;
 	const char *thumbname = NULL;
+	const char *audioname = NULL;
 	int ret;
 	unsigned int datalen;
 	char canon_path[300];
 
+	/* for debugging */
+	char *filetype;
+	char buf[32];
+
 	/* put complete canon path into canon_path */
 	ret = snprintf (canon_path, sizeof (canon_path) - 3, "%s\\%s",
 			gphoto2canonpath (camera, folder, context), filename);
 	if (ret < 0) {
 		gp_context_error (context,
-				  "Internal error #1 in get_file_func() (%s line %i)",
+				  _("Internal error #1 in get_file_func() (%s line %i)"),
 				  __FILE__, __LINE__);
 		return GP_ERROR;
 	}
 
+	/* for debugging */
+	switch (type) {
+	case GP_FILE_TYPE_PREVIEW: filetype = "thumbnail"; break;
+	case GP_FILE_TYPE_EXIF: filetype = "exif data"; break;
+	case GP_FILE_TYPE_AUDIO: filetype = "audio annotation"; break;
+	case GP_FILE_TYPE_NORMAL: filetype = "file itself"; break;
+	default: snprintf(buf, sizeof(buf), "unknown type %d", type); filetype = buf; break;
+	}
 	GP_DEBUG ("get_file_func: folder '%s' filename '%s' (i.e. '%s'), getting %s",
-		  folder, filename, canon_path,
-		  (type == GP_FILE_TYPE_PREVIEW) ? "thumbnail" : "file itself");
+		  folder, filename, canon_path, filetype);
 
-	/* preparation, if it is _PREVIEW or _EXIF we get the thumbnail name */
-	if (type == GP_FILE_TYPE_PREVIEW || type == GP_FILE_TYPE_EXIF) {
-		thumbname = canon_int_filename2thumbname (camera, canon_path);
-		if (thumbname == NULL) {
-			/* no thumbnail available */
+	/* preparation, if it is _AUDIO we convert the file name to
+	 * the corresponding audio file name. If the name of the audio
+	 * file is supplied, this step will be a no-op. */
+	if (type == GP_FILE_TYPE_AUDIO) {
+		audioname = canon_int_filename2audioname (camera, canon_path);
+		if (audioname == NULL) {
 			gp_context_error (context,
-					  "No thumbnail could be fould for %s",
+					  _("No audio file could be found for %s"),
 					  canon_path);
-			ret = GP_ERROR;
-		} 
+			return(GP_ERROR_FILE_NOT_FOUND);
+		}
 	}
 
-	/* fetch image/thumbnail/exif */	
+	/* fetch file/thumbnail/exif/audio/whatever */
 	switch (type) {
 		case GP_FILE_TYPE_NORMAL:
 			ret = canon_int_get_file (camera, canon_path, &data, &datalen,
@@ -411,18 +541,38 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 							       attr, context);
 			}
 			break;
+
+		case GP_FILE_TYPE_AUDIO:
+			if (*audioname != '\0') {
+				/* extra audio file */
+				ret = canon_int_get_file (camera, audioname, &data, &datalen,
+							  context);
+			} else {
+				/* internal audio file; not handled yet */
+				ret = GP_ERROR_NOT_SUPPORTED;
+			}
+			break;
+
 		case GP_FILE_TYPE_PREVIEW:
+			thumbname = canon_int_filename2thumbname (camera, canon_path);
+			if (thumbname == NULL) {
+				/* no thumbnail available */
+				GP_DEBUG (_("%s is a file type for which no thumbnail is provided"),canon_path);
+				return (GP_ERROR_NOT_SUPPORTED);
+			}
 #ifdef HAVE_EXIF
-			/* Check if we have libexif, it is an image and it is not
-			 * a PS A70 (which does not support EXIF), return not
+			/* Check if we have libexif, it is a jpeg file
+			 * and it is not a PowerShot Pro 70 (which
+			 * does not support EXIF), return not
 			 * supported here so that gPhoto2 query for
 			 * GP_FILE_TYPE_EXIF instead
 			 */
-			if (is_image (filename))
-				if (camera->pl->md->model != CANON_PS_A70) {
+			if (is_jpeg (filename)) {
+				if (camera->pl->md->model != CANON_CLASS_2) {
 					GP_DEBUG ("get_file_func: preview requested where "
 						  "EXIF should be possible");
 					return (GP_ERROR_NOT_SUPPORTED);
+				}
 			}
 #endif /* HAVE_EXIF */
 			if (*thumbname == '\0') {
@@ -435,12 +585,20 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 							  context);
 			}
 			break;
+
 		case GP_FILE_TYPE_EXIF:
 #ifdef HAVE_EXIF
-			/* the PS A70 does not support EXIF */
-			if (camera->pl->md->model == CANON_PS_A70)
+			/* the PowerShot Pro 70 does not support EXIF */
+			if (camera->pl->md->model == CANON_CLASS_2)
 				return (GP_ERROR_NOT_SUPPORTED);
-				
+
+			thumbname = canon_int_filename2thumbname (camera, canon_path);
+			if (thumbname == NULL) {
+				/* no thumbnail available */
+				GP_DEBUG (_("%s is a file type for which no thumbnail is provided"), canon_path);
+				return (GP_ERROR_NOT_SUPPORTED);
+			}
+
 			if (*thumbname == '\0') {
 				/* file internal thumbnail with EXIF data */
 				ret = canon_int_get_thumbnail (camera, canon_path, &data,
@@ -454,7 +612,8 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			GP_DEBUG ("get_file_func: EXIF file type requested but no libexif");
 			return (GP_ERROR_NOT_SUPPORTED);
 #endif /* HAVE_EXIF */
-			break;		
+			break;
+
 		default:
 			GP_DEBUG ("get_file_func: unsupported file type %i", type);
 			return (GP_ERROR_NOT_SUPPORTED);
@@ -480,7 +639,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	}
 
 	/* do different things with the data fetched above */
-	
+	/* FIXME: For which file type(s) should we gp_file_set_name(file,filename) ? */
 	switch (type) {
 		case GP_FILE_TYPE_PREVIEW:
 			/* Either this camera model does not support EXIF,
@@ -490,7 +649,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			 * in data.
 			 */
 			ret = canon_int_extract_jpeg_thumb (data, datalen, &thumbdata, &datalen, context);
-			
+
 			if (thumbdata != NULL) {
 				/* free old data */
 				free (data);
@@ -511,6 +670,13 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			gp_file_set_mime_type (file, GP_MIME_JPEG);	/* always */
 			gp_file_set_name (file, filename);
 			break;
+
+		case GP_FILE_TYPE_AUDIO:
+			gp_file_set_mime_type (file, GP_MIME_WAV);
+			gp_file_set_data_and_size (file, data, datalen);
+			gp_file_set_name (file, filename);
+			break;
+
 		case GP_FILE_TYPE_NORMAL:
 			gp_file_set_mime_type (file, filename2mimetype (filename));
 			gp_file_set_data_and_size (file, data, datalen);
@@ -521,7 +687,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			gp_file_set_mime_type (file, GP_MIME_JPEG);
 			gp_file_set_data_and_size (file, data, datalen);
 			break;
-#endif /* HAVE_EXIF */			
+#endif /* HAVE_EXIF */
 		default:
 			/* this case should've been caught above anyway */
 			if (data)
@@ -629,8 +795,8 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *context)
 	tm = localtime (&tmp_time);
 #ifdef HAVE_TM_GMTOFF
 	local_time = tmp_time + tm->tm_gmtoff;
-	GP_DEBUG ("camera_summary: converted %i to localtime %i (tm_gmtoff is %i)", 
-		  tmp_time, local_time, tm->tm_gmtoff);
+	GP_DEBUG ("camera_summary: converted %ld to localtime %ld (tm_gmtoff is %ld)",
+		  (long)tmp_time, (long)local_time, (long)tm->tm_gmtoff);
 #else
 	local_time = tmp_time - timezone;
 	GP_DEBUG ("camera_summary: converted %i to localtime %i (timezone is %i)",
@@ -700,7 +866,7 @@ delete_file_func (CameraFilesystem *fs, const char *folder, const char *filename
 	if (check_readiness (camera, context) != 1)
 		return GP_ERROR;
 
-	if (camera->pl->md->model == CANON_PS_A5 || camera->pl->md->model == CANON_PS_A5_ZOOM) {
+	if (camera->pl->md->model == CANON_CLASS_3) {
 		GP_DEBUG ("delete_file_func: deleting "
 			  "pictures disabled for cameras: PowerShot A5, PowerShot A5 ZOOM");
 
@@ -851,11 +1017,11 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, void
 		return GP_ERROR;
 
 	gp_camera_get_abilities (camera, &a);
-	if (camera->pl->speed > 57600 && (!strcmp (a.model, "Canon PowerShot A50")
-					  || !strcmp (a.model, "Canon PowerShot Pro70"))) {
+	/* Special case for A50 and Pro 70 */
+	if ((camera->pl->speed > 57600) && ((camera->pl->md->model == CANON_CLASS_1)
+					    || (camera->pl->md->model == CANON_CLASS_2))) {
 		gp_context_error (context,
-				  _
-				  ("Speeds greater than 57600 are not supported for uploading to this camera"));
+				  _("Speeds greater than 57600 are not supported for uploading to this camera"));
 		return GP_ERROR_NOT_SUPPORTED;
 	}
 
@@ -915,8 +1081,7 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, void
 				    dirnum = atoi (buf);
 				    if (dirnum == 999 && !camera->pl->upload_keep_filename) {
 					    gp_context_error (context,
-							      _
-							      ("Could not upload, no free folder name available!\n"
+							      _("Could not upload, no free folder name available!\n"
 							       "999CANON folder name exists and has an AUT_9999.JPG picture in it."));
 					    return GP_ERROR;
 				    } else {
@@ -980,11 +1145,11 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, void
 		return GP_ERROR;
 
 	gp_camera_get_abilities (camera, &a);
-	if (camera->pl->speed > 57600 && (!strcmp (a.model, "Canon PowerShot A50")
-					  || !strcmp (a.model, "Canon PowerShot Pro70"))) {
+	/* Special case for A50 and Pro 70 */
+	if ((camera->pl->speed > 57600) && ((camera->pl->md->model == CANON_CLASS_1)
+					    || (camera->pl->md->model == CANON_CLASS_2))) {
 		gp_context_error (context,
-				  _
-				  ("Speeds greater than 57600 are not supported for uploading to this camera"));
+				  _("Speeds greater than 57600 are not supported for uploading to this camera"));
 		return GP_ERROR_NOT_SUPPORTED;
 	}
 
@@ -1024,8 +1189,7 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, void
 				dirnum = atoi (buf);
 				if (dirnum == 999) {
 					gp_context_error (context,
-							  _
-							  ("Could not upload, no free folder name available!\n"
+							  _("Could not upload, no free folder name available!\n"
 							   "999CANON folder name exists and has an AUT_9999.JPG picture in it."));
 					return GP_ERROR;
 				} else {
@@ -1071,6 +1235,8 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, void
 
 /****************************************************************************/
 
+/* Get configuration into screen widgets for display. */
+
 static int
 camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 {
@@ -1158,6 +1324,7 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	return GP_OK;
 }
 
+/* Set camera configuration from screen widgets. */
 static int
 camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 {
@@ -1197,14 +1364,14 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 	if (gp_widget_changed (w)) {
 		/* XXXXX mark CameraFS as dirty */
 		gp_widget_get_value (w, &camera->pl->list_all_files);
-		GP_DEBUG ("New config value for tmb: %i", &camera->pl->list_all_files);
+		GP_DEBUG ("New config value for tmb: %i", camera->pl->list_all_files);
 	}
 	
 #ifdef CANON_EXPERIMENTAL_UPLOAD
 	gp_widget_get_child_by_label (window, _("Keep filename on upload"), &w);
 	if (gp_widget_changed (w)) {
 		gp_widget_get_value (w, &camera->pl->upload_keep_filename);
-		GP_DEBUG ("New config value for tmb: %i", &camera->pl->upload_keep_filename);
+		GP_DEBUG ("New config value for tmb: %i", camera->pl->upload_keep_filename);
 	}
 #endif /* CANON_EXPERIMENTAL_UPLOAD */
 
@@ -1233,8 +1400,10 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		strcpy (info->file.type, GP_MIME_AVI);
 	else if (is_image (filename))
 		strcpy (info->file.type, GP_MIME_JPEG);
+	else if (is_audio (filename))
+		strcpy (info->file.type, GP_MIME_WAV);
 	else
-		/* May no be correct behaviour ... */
+		/* May not be correct behaviour ... */
 		strcpy (info->file.type, "unknown");
 
 	strcpy (info->file.name, filename);
@@ -1329,10 +1498,13 @@ remove_dir_func (CameraFilesystem *fs, const char *folder, const char *name, voi
  * @camera: the camera to initialize
  * @context: a #GPContext
  *
- * This routine initializes the serial/USB port and also load the
+ * This routine initializes the serial/USB port and also loads the
  * camera settings. Right now it is only the speed that is
  * saved.
- **/
+ *
+ * Returns: gphoto2 error code
+ *
+ */
 int
 camera_init (Camera *camera, GPContext *context)
 {
@@ -1342,11 +1514,8 @@ camera_init (Camera *camera, GPContext *context)
 
 	/* First, set up all the function pointers */
 	camera->functions->exit = camera_exit;
-#ifdef CANON_EXPERIMENTAL_CAPTURE
-	/* we should only set this if the camera supports it */
-	camera->functions->capture_preview = camera_capture_preview;
 	camera->functions->capture = camera_capture;
-#endif /* CANON_EXPERIMENTAL_CAPTURE */
+	camera->functions->capture_preview = camera_capture_preview;
 	camera->functions->get_config = camera_get_config;
 	camera->functions->set_config = camera_set_config;
 	camera->functions->summary = camera_summary;
@@ -1374,11 +1543,6 @@ camera_init (Camera *camera, GPContext *context)
 	camera->pl->seq_tx = 1;
 	camera->pl->seq_rx = 1;
 
-#ifdef CANON_EXPERIMENTAL_CAPTURE
-	/* we are currently not capturing, are we? */
-	camera->pl->capturing = FALSE;
-#endif
-
 	/* default to false, i.e. list only known file types, use DCIF filenames */
 	camera->pl->list_all_files = FALSE;
 #ifdef CANON_EXPERIMENTAL_UPLOAD
