@@ -2,7 +2,7 @@
  *
  * File: util.h
  *
- * $Id: util.h,v 1.13 2002/01/12 23:31:42 lutz Exp $
+ * $Id: util.h,v 1.16 2003/02/27 14:52:08 swestin Exp $
  ****************************************************************************/
 
 #ifndef _CANON_UTIL_H
@@ -23,9 +23,10 @@
  *
  ****************************************************************************/
 
-void dump_hex(Camera *camera,const char *msg, const unsigned char *buf, int len);
+/*void dump_hex(Camera *camera,const char *msg, const unsigned char *buf, int len);*/
 
 int is_thumbnail (const char *name);
+int is_audio (const char *name);
 int is_image (const char *name);
 int is_movie (const char *name);
 int is_jpeg (const char *name);
