@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2000-2001, Brian Beattie <beattie@aracnet.com>, et. al.
+* Copyright  2000-2001, Brian Beattie <beattie@aracnet.com>, et. al.
 *
 *	This software was created with the help of proprietary
 *      information belonging to StarDot Technologies
@@ -17,9 +17,9 @@
 *		* Add checksum to mesa_read_row()			  GDB
 *		* Add return with number of bytes to mesa_read_image()	  GDB
 *									  GDB
-* $Id: mesalib.c,v 1.13 2002/01/22 04:57:59 dfandrich Exp $
+* $Id: mesalib.c,v 1.15 2002/12/15 11:43:24 asl Exp $
 */
-#include <config.h>
+#include "config.h"
 
 #include "mesalib.h"
 
