@@ -1,6 +1,6 @@
 /* directory.c
  *
- * Copyright (C) 2001 Lutz Mller <urc8@rz.uni-karlsruhe.de>
+ * Copyright  2001 Lutz Mller <lutz@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include <config.h>
+#include "config.h"
 
 #include <errno.h>
 #include <stdio.h>
@@ -122,9 +122,11 @@ int camera_abilities (CameraAbilitiesList *list)
 #endif
 #ifdef DEBUG
         a.file_operations = GP_FILE_OPERATION_PREVIEW |
-			    GP_FILE_OPERATION_DELETE;
+			    GP_FILE_OPERATION_DELETE | 
+			    GP_FILE_OPERATION_EXIF;
 #else
-	a.file_operations = GP_FILE_OPERATION_DELETE;
+	a.file_operations = GP_FILE_OPERATION_DELETE |
+			    GP_FILE_OPERATION_EXIF;
 #endif
         a.folder_operations = GP_FOLDER_OPERATION_MAKE_DIR |
 			      GP_FOLDER_OPERATION_REMOVE_DIR |
@@ -171,8 +173,10 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		/* Give some feedback */
 		gp_context_progress_update (context, id, n + 1);
 		gp_context_idle (context);
-		if (gp_context_cancel (context) == GP_CONTEXT_FEEDBACK_CANCEL)
+		if (gp_context_cancel (context) == GP_CONTEXT_FEEDBACK_CANCEL) {
+			GP_SYSTEM_CLOSEDIR (dir);
 			return (GP_ERROR_CANCEL);
+		}
 
 		if (strcmp(GP_SYSTEM_FILENAME(de), "." ) &&
 		    strcmp(GP_SYSTEM_FILENAME(de), "..")) {
@@ -183,6 +187,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		}
 		n++;
 	}
+	GP_SYSTEM_CLOSEDIR (dir);
 	gp_context_progress_stop (context, id);
 
 	return (GP_OK);
@@ -252,9 +257,10 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		/* Give some feedback */
 		gp_context_progress_update (context, id, n + 1);
 		gp_context_idle (context);
-		if (gp_context_cancel (context) == GP_CONTEXT_FEEDBACK_CANCEL)
+		if (gp_context_cancel (context) == GP_CONTEXT_FEEDBACK_CANCEL) {
+			GP_SYSTEM_CLOSEDIR (dir);
 			return (GP_ERROR_CANCEL);
-
+		}
 		if ((strcmp (GP_SYSTEM_FILENAME (de), "." ) != 0) &&
 		    (strcmp (GP_SYSTEM_FILENAME (de), "..") != 0)) {
 			sprintf (buf, "%s%s", f, GP_SYSTEM_FILENAME (de));
@@ -272,6 +278,7 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		}
 		n++;
 	}
+	GP_SYSTEM_CLOSEDIR (dir);
 	gp_context_progress_stop (context, id);
 
 	return (GP_OK);
