@@ -1,7 +1,7 @@
 /****************************************************************/
 /* library.c  - Gphoto2 library for the KBGear JamCam v2 and v3 */
 /*                                                              */
-/* Copyright (C) 2001 Chris Pinkham                             */
+/* Copyright  2001 Chris Pinkham                             */
 /*                                                              */
 /* Author: Chris Pinkham <cpinkham@infi.net>                    */
 /*                                                              */
@@ -22,10 +22,11 @@
 /* Free Software Foundation, Inc., 59 Temple Place - Suite 330, */
 /* Boston, MA 02111-1307, USA.                                  */
 /****************************************************************/
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 #include <gphoto2.h>
 #include <time.h>
 #include <unistd.h>
@@ -81,7 +82,7 @@ static int jamcam_get_int_at_pos( unsigned char *buf, int pos ) {
 }
 
 static int jamcam_set_usb_mem_pointer( Camera *camera, int position ) {
-	char reply[4];
+	char reply[8];
 
 	GP_DEBUG ("* jamcam_set_usb_mem_pointer");
 	GP_DEBUG ("*** position:  %d (0x%x)",
@@ -97,7 +98,7 @@ static int jamcam_set_usb_mem_pointer( Camera *camera, int position ) {
 		0xa0,
 		0,
 		0,
-		reply, 4 );
+		reply, 8 );
 
 	return( GP_OK );
 }
@@ -401,9 +402,10 @@ int jamcam_request_image( Camera *camera, CameraFile *file,
 		char *buf, int *len, int number, GPContext *context ) {
 	int position;
 	int result;
-	char tmp_buf[300000];
+	char *tmp_buf;
 
 	GP_DEBUG ("* jamcam_request_image");
+	tmp_buf = malloc(640*480*3);
 
 	position = jamcam_files[number].position;
 
@@ -419,7 +421,7 @@ int jamcam_request_image( Camera *camera, CameraFile *file,
 			0x0000,
 			NULL, 0 );
 	}
-
+	
 	result = jamcam_fetch_memory( camera, file, tmp_buf, position,
 		jamcam_files[number].data_incr, context );
 
@@ -436,12 +438,14 @@ int jamcam_request_image( Camera *camera, CameraFile *file,
 		*len = jamcam_files[number].width * jamcam_files[number].height;
 		memcpy( buf, tmp_buf + 0x10, *len );
 	}
+	free (tmp_buf);
 
 	return( result );
 }
 
 struct jamcam_file *jamcam_file_info(Camera *camera, int number)
 {
+	GP_DEBUG(" * jamcam_file_info, nr is %d", number);
 	return( &jamcam_files[number] );
 }
 
@@ -487,7 +491,6 @@ int jamcam_request_thumbnail( Camera *camera, CameraFile *file,
 		}
 	}
 
-
 	/* fetch thumbnail lines and build the thumbnail */
 	position += 10 * jamcam_files[number].width;
 	id = gp_context_progress_start (context, 60.,
