@@ -1,4 +1,4 @@
-#include <config.h>
+#include "config.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -12,7 +12,7 @@
 #  include <libintl.h>
 #  undef _
 #  ifdef gettext_noop
-#    define _(String) dgettext (GETTEXT_PACKAGE, String)
+#    define _(String) dgettext (PACKAGE, String)
 #    define N_(String) gettext_noop (String)
 #  else
 #    define _(String) (String)
@@ -40,8 +40,8 @@ int camera_abilities (CameraAbilitiesList *list) {
 	CameraAbilities a;
 
 	memset (&a, 0, sizeof(a));
-	strcpy(a.model, "Kodak DC210");
-	a.status = GP_DRIVER_STATUS_EXPERIMENTAL;
+	strcpy(a.model, "Kodak:DC210");
+	a.status = GP_DRIVER_STATUS_TESTING;
 	a.port     = GP_PORT_SERIAL;
 	a.speed[0] = 9600;
 	a.speed[1] = 19200;
@@ -445,7 +445,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 static int camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path, GPContext *context) {
 
 
-	if (type && type != GP_OPERATION_CAPTURE_IMAGE){
+	if (type && type != GP_CAPTURE_IMAGE){
 		DC210_DEBUG("Unsupported action 0x%.2X\n", type);
 		return (GP_ERROR_NOT_SUPPORTED);
 	};
