@@ -1,6 +1,6 @@
 /* library.c
  *
- * Copyright (C) 2001 Lutz Mller
+ * Copyright  2001 Lutz Mller
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,7 +38,7 @@
 #    define N_(String) gettext_noop (String)
 #  else
 #    define N_(String) (String)
-#  endif 
+#  endif
 #else
 #  define textdomain(String) (String)
 #  define gettext(String) (String)
@@ -226,7 +226,8 @@ camera_abilities (CameraAbilitiesList* list)
                                GP_OPERATION_CAPTURE_IMAGE |
                                GP_OPERATION_CAPTURE_PREVIEW;
                 a.file_operations = GP_FILE_OPERATION_DELETE |
-                                    GP_FILE_OPERATION_PREVIEW;
+                                    GP_FILE_OPERATION_PREVIEW |
+				    GP_FILE_OPERATION_EXIF;
                 a.folder_operations = GP_FOLDER_OPERATION_DELETE_ALL;
                 gp_abilities_list_append (list, a);
         }
@@ -313,7 +314,7 @@ static int
 test_speed (Camera *camera, GPContext *context)
 {
 	int i;
-	unsigned int speeds[] = {115200, 9600, 57600, 38400, 19200, 
+	unsigned int speeds[] = {115200, 9600, 57600, 38400, 19200,
 				 4800, 2400, 1200, 600, 300};
 	unsigned int id;
 	GPPortSettings settings;
@@ -668,7 +669,7 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
         if (status.date.year > 80) year_4_digits = status.date.year + 1900;
         else year_4_digits = status.date.year + 2000;
         tm_struct.tm_year = year_4_digits - 1900;
-        tm_struct.tm_mon = status.date.month;
+        tm_struct.tm_mon = status.date.month - 1;
         tm_struct.tm_mday = status.date.day;
         tm_struct.tm_hour = status.date.hour;
         tm_struct.tm_min = status.date.minute;
@@ -882,7 +883,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
                 xtime = i;
                 tm_struct = localtime (&xtime);
                 date.year   = tm_struct->tm_year - 100;
-                date.month  = tm_struct->tm_mon;
+                date.month  = tm_struct->tm_mon + 1;
                 date.day    = tm_struct->tm_mday;
                 date.hour   = tm_struct->tm_hour;
                 date.minute = tm_struct->tm_min;
@@ -1127,7 +1128,7 @@ localization_file_read (Camera *camera, const char *file_name,
                 default:
                         /****************************************/
                         /* j == 0: We have to read the second   */
-                        /*         half of the byte to send.    */
+                        /* half of the byte to send.            */
                         /* j == 1: We'll compose our byte.      */
                         /****************************************/
                         if ((f != '0') && (f != '1') && (f != '2') &&
@@ -1247,9 +1248,9 @@ camera_init (Camera* camera, GPContext *context)
                 /* Initiate the connection */
 		C(speed = test_speed (camera, context));
 #if 0
-//Ideally, we need to reset the speed to the speed that we encountered
-//after each operation (multiple programs accessing the camera). However,
-//that takes quite a bit of time for HP cameras...
+/* Ideally, we need to reset the speed to the speed that we encountered
+   after each operation (multiple programs accessing the camera). However,
+   that takes quite a bit of time for HP cameras... */
 		camera->pl->speed = speed;
 #endif
 
