@@ -1,7 +1,7 @@
 /* largan.c
  *
- * Copyright (C) 2001 Lutz Mller <urc8@rz.uni-karlsruhe.de>
- * Copyright (C) 2002 Hubert Figuiere <hfiguiere@teaser.fr>
+ * Copyright  2001 Lutz Mller <lutz@users.sourceforge.net>
+ * Copyright  2002 Hubert Figuiere <hfiguiere@teaser.fr>
  * Code largely borrowed to lmini-0.1 by Steve O Connor
  *
  * This library is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
  * Currently only largan lmini is supported
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -50,13 +50,15 @@
 #  define N_(String) (String)
 #endif
 
+#define TIMEOUT 1500
+
 /* internal functions prototypes */
 static uint8_t convert_name_to_index (const char * name);
 
 int
 camera_id (CameraText *id) 
 {
-	strcpy(id->text, "largan");
+	strcpy(id->text, "largan-lmini");
 
 	return (GP_OK);
 }
@@ -68,13 +70,13 @@ struct largan_cameras {
 	unsigned short idProduct;
 	char hasSerial;
 } largan_cameras [] = {
-	{ "Largan Lmini", 0, 0, 1 },
+	{ "Largan:Lmini", 0, 0, 1 },
 /* these are other largan camera. I'm not sure they use the same
  * protocol */
-/*	{ "Largan Chameleon Slim", 0, 0, 0 },
-	{ "Largan Chameleon Mega", 0, 0, 0 },
-	{ "Largan Chameleon", 0, 0, 0 },
-	{ "Largan Easy 800", 0, 0, 1 }, */
+/*	{ "Largan:Chameleon Slim", 0, 0, 0 },
+	{ "Largan:Chameleon Mega", 0, 0, 0 },
+	{ "Largan:Chameleon", 0, 0, 0 },
+	{ "Largan:Easy 800", 0, 0, 1 }, */
 	{ NULL, 0, 0, 0 }
 };
 
@@ -108,7 +110,7 @@ camera_abilities (CameraAbilitiesList *list)
 		a.folder_operations = 	GP_FOLDER_OPERATION_NONE;
 	
 		if (a.port) {
-			// only append if port is defined.
+			/* only append if port is defined.*/
 			gp_abilities_list_append(list, a);
 		}
 	}
@@ -184,6 +186,24 @@ convert_name_to_index (const char * name)
 }
 
 
+static int
+delete_file_func (CameraFilesystem *fs, const char *folder,
+		  const char *filename, void *data, GPContext *context)
+{
+	Camera *camera = data;
+	int ret;
+	int index;
+	
+	/* Delete one file from the camera. 
+	(only the last file can be deleted)*/
+	
+	index = convert_name_to_index (filename);
+	ret = largan_erase (camera, index);
+	
+	return ret;
+}
+
+
 static int
 delete_all_func (CameraFilesystem *fs, const char *folder, void *data,
 		 GPContext *context)
@@ -195,29 +215,30 @@ delete_all_func (CameraFilesystem *fs, const char *folder, void *data,
 	 * such a functionality, just don't implement this function.
 	 */
 
-	ret = largan_erase (camera, 1);
+	ret = largan_erase (camera, 0xff);
 	
 	return ret;
 }
 
+#if 0
 static int
 camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 {
-	int ret;
-	/*
-	 * Capture a preview and return the data in the given file (again,
-	 * use gp_file_set_data_and_size, gp_file_set_mime_type, etc.).
-	 * libgphoto2 assumes that previews are NOT stored on the camera's 
-	 * disk. If your camera does, please delete it from the camera.
-	 */
-	ret = largan_capture (camera);
-	if (ret != GP_OK) {
-		return ret;
-	}
-
+ 	int ret;
+ 	/*
+ 	 * Capture a preview and return the data in the given file (again,
+ 	 * use gp_file_set_data_and_size, gp_file_set_mime_type, etc.).
+ 	 * libgphoto2 assumes that previews are NOT stored on the camera's 
+ 	 * disk. If your camera does, please delete it from the camera.
+ 	*/ 
+ 	ret = largan_capture (camera);
+ 	if (ret != GP_OK) {
+ 		return ret;
+ 	}
 
 	return (GP_OK);
 }
+#endif
 
 static int
 camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
@@ -276,7 +297,7 @@ static int
 get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	       CameraFileInfo *info, void *data, GPContext *context)
 {
-	Camera *camera = data;
+	/* Camera *camera = data; */
 
 	/* Get the file info here and write it into <info> */
 
@@ -315,7 +336,7 @@ camera_init (Camera *camera, GPContext *context)
         /* First, set up all the function pointers */
         camera->functions->exit                 = camera_exit;
         camera->functions->capture              = camera_capture;
-        camera->functions->capture_preview      = camera_capture_preview;
+	/*        camera->functions->capture_preview      = camera_capture_preview;*/
         camera->functions->summary              = camera_summary;
         camera->functions->manual               = camera_manual;
         camera->functions->about                = camera_about;
@@ -325,8 +346,8 @@ camera_init (Camera *camera, GPContext *context)
 				      NULL, camera);
 	gp_filesystem_set_info_funcs (camera->fs, get_info_func, NULL,
 				      camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-				      NULL, camera);
+	gp_filesystem_set_file_funcs (camera->fs, get_file_func, 
+				      delete_file_func, camera);
 	gp_filesystem_set_folder_funcs (camera->fs, NULL,
 					delete_all_func, NULL, NULL, camera);
 
@@ -344,13 +365,17 @@ camera_init (Camera *camera, GPContext *context)
 		/* Remember the selected speed */
 		selected_speed = settings.serial.speed;
 
-		settings.serial.speed    = 19200;// initial speed is 19200
+		settings.serial.speed    = 19200;/* initial speed is 19200 */
 		settings.serial.bits     = 8;
 		settings.serial.parity   = 0;
 		settings.serial.stopbits = 1;
+		ret = gp_port_set_timeout (camera->port, TIMEOUT);
+		if (ret < 0) {
+			return (ret);
+		}
 		break;
 	case GP_PORT_USB:
-		// TODO check with a USB camera....
+		/* TODO check with a USB camera.... */
 		settings.usb.inep       = 0x82;
 		settings.usb.outep      = 0x01;
 		settings.usb.config     = 1;
@@ -365,11 +390,6 @@ camera_init (Camera *camera, GPContext *context)
 	if (ret < 0)
 		return (ret);
 
-	/*
-	ret = gp_port_set_timeout (camera->port, TIMEOUT);
-	if (ret < 0)
-		return (ret);
-*/
 	/*
 	 * Once you have configured the port, you should check if a 
 	 * connection to the camera can be established.
