@@ -19,7 +19,7 @@
 *                                                                     *
 **********************************************************************/
 
-/* $Id: download.c,v 1.15 2002/12/15 11:43:26 asl Exp $ */
+/* $Id: download.c,v 1.16 2004/08/22 22:34:26 marcusmeissner Exp $ */
 
 #include "config.h"
 
@@ -309,7 +309,7 @@ int dimagev_get_thumbnail(dimagev_t *dimagev, int file_number, CameraFile *file)
 
 		free(r);
 
-		GP_DEBUG( "dimagev_get_thumbnail::current file size is %d", size);
+		GP_DEBUG( "dimagev_get_thumbnail::current file size is %ld", size);
 	}
 
 	size++;
