@@ -1,7 +1,7 @@
 /* mdc800.c
  *
- * Copyright (C) 1999/2000 by Henning Zabel <henning@uni-paderborn.de>
- * Copyright (C) 2001 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright  1999/2000 by Henning Zabel <henning@uni-paderborn.de>
+ * Copyright  2001 Marcus Meissner <marcus@jet.franken.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -54,7 +54,7 @@ camera_abilities (CameraAbilitiesList *list)
 	CameraAbilities a;
 
 	memset(&a,0,sizeof(a));
-	strcpy(a.model,	"Mustek MDC 800");
+	strcpy(a.model,	"Mustek:MDC 800");
 	a.status	= GP_DRIVER_STATUS_EXPERIMENTAL;
 	a.port		= GP_PORT_SERIAL | GP_PORT_USB;
 	a.speed[0]	= 19200;
@@ -69,15 +69,19 @@ camera_abilities (CameraAbilitiesList *list)
 	a.file_operations   = 	GP_FILE_OPERATION_DELETE |
 				GP_FILE_OPERATION_PREVIEW;
 	a.folder_operations = 	GP_FOLDER_OPERATION_NONE;
+/* Deliberately commented out. this driver does not work due to USB interrupts
 	gp_abilities_list_append(list, a);
+ */
 	return (GP_OK);
 }
 
+#if 0
 static int
 camera_exit (Camera *camera, GPContext *context)
 {
 	return (GP_OK);
 }
+#endif
 
 static int
 get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
@@ -85,11 +89,9 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	       GPContext *context)
 {
 	Camera *camera = data;
-	int	nr, result;
-	long int size;
+	int	size, nr, result;
 
 	nr = gp_filesystem_number(fs, folder, filename, context);
-
 	if(nr < 0)
 		return nr;
 
@@ -100,10 +102,10 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		break;
 #endif
 	case GP_FILE_TYPE_NORMAL:
-		result = mdc800_getImage(camera,nr,&data,(int*)&size);
+		result = mdc800_getImage(camera,nr,&data,&size);
 		break;
 	case GP_FILE_TYPE_PREVIEW:
-		result = mdc800_getThumbnail(camera,nr,&data,(int*)&size);
+		result = mdc800_getThumbnail(camera,nr,&data,&size);
 		break;
 	default:
 		return (GP_ERROR_NOT_SUPPORTED);
@@ -121,7 +123,7 @@ static int
 put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 	       void *data, GPContext *context)
 {
-/*	Camera *camera;*/
+/* Camera *camera;*/
 
 	/*
 	 * Upload the file to the camera. Use gp_file_get_data_and_size,
@@ -191,6 +193,7 @@ camera_config_set (Camera *camera, CameraWidget *window, GPContext *ctx)
 	return (GP_OK);
 }
 
+#if 0
 static int
 camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 {
@@ -203,6 +206,7 @@ camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 
 	return (GP_OK);
 }
+#endif
 
 static int
 camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
@@ -319,7 +323,7 @@ int
 camera_init (Camera *camera, GPContext *context)
 {
         /* First, set up all the function pointers */
-        // camera->functions->exit                 = camera_exit;
+        /* camera->functions->exit                 = camera_exit; */
         camera->functions->get_config		= camera_config_get;
         camera->functions->set_config		= camera_config_set;
         camera->functions->capture              = camera_capture;
