@@ -22,10 +22,12 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <time.h>
 
 #include <gphoto2-library.h>
 #include <gphoto2-port-log.h>
 
+
 #ifdef ENABLE_NLS
 #  include <libintl.h>
 #  undef _
@@ -166,7 +168,7 @@ report_result (GPContext *context, short result)
 
 	for (i = 0; ptp_errors[i].txt; i++)
 		if (ptp_errors[i].n == result)
-			gp_context_error (context, dgettext(GETTEXT_PACKAGE, ptp_errors[i].txt));
+			gp_context_error (context, "%s", dgettext(GETTEXT_PACKAGE, ptp_errors[i].txt));
 }
 
 static int
@@ -192,7 +194,7 @@ translate_gp_result (int result)
 		return (PTP_RC_OK);
 	case GP_ERROR:
 	default:
-		GP_DEBUG ("PTP: gp_port_* function returned 0x%4x \t %i",result,result);
+		GP_DEBUG ("PTP: gp_port_* function returned 0x%04x \t %i",result,result);
 		return (PTP_RC_GeneralError);
 	}
 }
@@ -226,9 +228,12 @@ static struct {
 	{"Kodak:DX4330", 0x040a, 0x0555},
 	{"Kodak:CX4200", 0x040a, 0x0560},
 	{"Kodak:CX4210", 0x040a, 0x0565},
+	{"Kodak:LS473", 0x040a, 0x0565},
 	/* both above with different product IDs
 	   normal/retail versions of the same model */
 	{"Kodak:CX4300", 0x040a, 0x0566},
+	{"Kodak:CX4310", 0x040a, 0x0566},
+	{"Kodak:LS753",  0x040a, 0x0567},
 	{"Kodak:LS443",  0x040a, 0x0568},
 	{"Kodak:LS663",  0x040a, 0x0569},
 	{"Kodak:DX6340", 0x040a, 0x0570},
@@ -238,12 +243,23 @@ static struct {
 	{"Kodak:CX6200", 0x040a, 0x0574},
 	{"Kodak:DX6490", 0x040a, 0x0575},
 	{"Kodak:DX4530", 0x040a, 0x0576},
+	{"Kodak:DX7630", 0x040a, 0x0577},
+	{"Kodak:CX7300", 0x040a, 0x0578},
+	{"Kodak:CX7310", 0x040a, 0x0578},
+	{"Kodak:CX7220", 0x040a, 0x0579},
+	{"Kodak:CX7330", 0x040a, 0x057a},
+	{"Kodak:CX7430", 0x040a, 0x057b},
+	{"Kodak:CX7530", 0x040a, 0x057c},
+	{"Kodak:DX7440", 0x040a, 0x057d},
+	{"Kodak:CX6445", 0x040a, 0x0584},
 
 	/* HP PTP cameras */
 	{"HP:PhotoSmart 812 (PTP mode)", 0x03f0, 0x4202},
 	{"HP:PhotoSmart 850 (PTP mode)", 0x03f0, 0x4302},
 	/* HP PhotoSmart 935: T. Kaproncai, 25 Jul 2003*/
 	{"HP:PhotoSmart 935 (PTP mode)", 0x03f0, 0x4402},
+	/* HP:PhotoSmart 945: T. Jelbert, 2004/03/29	*/
+	{"HP:PhotoSmart 945 (PTP mode)", 0x03f0, 0x4502},
 	{"HP:PhotoSmart 318 (PTP mode)", 0x03f0, 0x6302},
 	{"HP:PhotoSmart 612 (PTP mode)", 0x03f0, 0x6302},
 	{"HP:PhotoSmart 715 (PTP mode)", 0x03f0, 0x6402},
@@ -252,26 +268,36 @@ static struct {
 	{"HP:PhotoSmart 720 (PTP mode)", 0x03f0, 0x6702},
 	{"HP:PhotoSmart 620 (PTP mode)", 0x03f0, 0x6802},
 	{"HP:PhotoSmart 735 (PTP mode)", 0x03f0, 0x6a02},
+	{"HP:PhotoSmart 635 (PTP mode)", 0x03f0, 0x7102},
 	/* report from Federico Prat Villar <fprat@lsi.uji.es> */
 	{"HP:PhotoSmart 43x (PTP mode)", 0x03f0, 0x7202},
 
-	/* I got information that all SONY PTP cameras use the same
-	   product/vendor IDs */
+	/* Most Sony PTP cameras use the same product/vendor IDs. */
+	{"Sony:PTP",                  0x054c, 0x004e},
 	{"Sony:DSC-P5 (PTP mode)",    0x054c, 0x004e},
+	{"Sony:DSC-P10 (PTP mode)",   0x054c, 0x004e},
 	{"Sony:DSC-F707V (PTP mode)", 0x054c, 0x004e},
+	{"Sony:DSC-F828 (PTP mode)",  0x054c, 0x004e},
 	{"Sony:DSC-P30 (PTP mode)",   0x054c, 0x004e},
 	/* P32 reported on May 1st by Justin Alexander <justin (at) harshangel.com> */
 	{"Sony:DSC-P31 (PTP mode)",   0x054c, 0x004e}, 
 	{"Sony:DSC-P32 (PTP mode)",   0x054c, 0x004e}, 
+	{"Sony:DSC-P41 (PTP mode)",   0x054c, 0x004e}, 
+	{"Sony:DSC-P43 (PTP mode)",   0x054c, 0x004e}, 
 	{"Sony:DSC-P50 (PTP mode)",   0x054c, 0x004e},
 	{"Sony:DSC-P52 (PTP mode)",   0x054c, 0x004e},
+	{"Sony:DSC-P71 (PTP mode)",   0x054c, 0x004e},
 	{"Sony:DSC-P72 (PTP mode)",   0x054c, 0x004e},
 	{"Sony:DSC-P92 (PTP mode)",   0x054c, 0x004e},
+	{"Sony:DSC-P93 (PTP mode)",   0x054c, 0x004e},
+	{"Sony:DSC-P100 (PTP mode)",  0x054c, 0x004e},
 	{"Sony:DSC-S75 (PTP mode)",   0x054c, 0x004e},
 	{"Sony:DSC-S85 (PTP mode)",   0x054c, 0x004e},
+	{"Sony:DSC-T1 (PTP mode)",    0x054c, 0x004e},
 	{"Sony:DSC-U20 (PTP mode)",   0x054c, 0x004e},
-	{"Sony:DSC-V1 (PTP mode)",   0x054c, 0x004e},
+	{"Sony:DSC-V1 (PTP mode)",    0x054c, 0x004e},
 	{"Sony:MVC-CD300 (PTP mode)", 0x054c, 0x004e},
+	{"Sony:MVC-CD500 (PTP mode)", 0x054c, 0x004e},
 
 	/* Nikon Coolpix 2500: M. Meissner, 05 Oct 2003 */
 	{"Nikon:Coolpix 2500 (PTP mode)", 0x04b0, 0x0109},
@@ -293,12 +319,24 @@ static struct {
 	{"Nikon:Coolpix 2100 (PTP mode)", 0x04b0, 0x0117},
 	/* Nikon Coolpix 5400: T. Kaproncai, 25 Jul 2003 */
 	{"Nikon:Coolpix 5400 (PTP mode)", 0x04b0, 0x0119},
+	/* Nikon Coolpix 3700: T. Ehlers, 18 Jan 2004 */
+	{"Nikon:Coolpix 3700 (PTP mode)", 0x04b0, 0x011d},
+	/* Nikon Coolpix 3200 */
+	{"Nikon:Coolpix 3200 (PTP mode)", 0x04b0, 0x0121},
+	/* Nikon Coolpix 2200 */
+	{"Nikon:Coolpix 2200 (PTP mode)", 0x04b0, 0x0122},
 	/* Nikon Coolpix SQ: M. Holzbauer, 07 Jul 2003 */
 	{"Nikon:Coolpix SQ (PTP mode)",   0x04b0, 0x0202},
+	/* lars marowski bree, 16.8.2004 */
+	{"Nikon:Coolpix 4200 (PTP mode)", 0x04b0, 0x0204},
 	/* Nikon Coolpix 2000 */
 	{"Nikon:Coolpix 2000 (PTP mode)", 0x04b0, 0x0302},
 	/* Nikon D100 has a PTP mode: westin 2002.10.16 */
 	{"Nikon:DSC D100 (PTP mode)",     0x04b0, 0x0402},
+	{"Nikon:DSC D70 (PTP mode)",      0x04b0, 0x0406},
+
+	/* in pictbridge mode */
+	{"Panasonic:DMC-FZ20",            0x04da, 0x2374},
 
 	/* From VICTOR <viaaurea@yahoo.es> */
 	{"Olympus:C-350Z",                0x07b4, 0x0114},
@@ -322,16 +360,44 @@ static struct {
 
 	{"Canon:PowerShot A70 (PTP)",           0x04a9, 0x3073},
 	{"Canon:PowerShot A60 (PTP)",           0x04a9, 0x3074},
-	{"Canon:PowerShot A80 (PTP)",           0x04a9, 0x309A},
 		/* IXUS 400 has the same PID in both modes, Till Kamppeter */
 	{"Canon:Digital IXUS 400 (PTP mode)",   0x04a9, 0x3075},
+	{"Canon:PowerShot S50 (PTP mode)",      0x04a9, 0x3077},
 	{"Canon:PowerShot G5 (PTP mode)",       0x04a9, 0x3085},
+	{"Canon:Elura 50 (PTP mode)",           0x04a9, 0x3087},
 		/* 0x3084 is the EOS 300D/Digital Rebel in normal (canon) mode */
 	{"Canon:EOS 300D (PTP mode)",           0x04a9, 0x3099},
 	{"Canon:EOS Digital Rebel (PTP mode)",  0x04a9, 0x3099},
 	{"Canon:EOS Kiss Digital (PTP mode)",   0x04a9, 0x3099},
-
+	{"Canon:PowerShot A80 (PTP)",           0x04a9, 0x309a},
 	{"Canon:Digital IXUS i (PTP mode)",     0x04a9, 0x309b},
+	{"Canon:PowerShot S1 IS (PTP mode)",    0x04a9, 0x309c},
+	{"Canon:Powershot S60 (PTP mode)",      0x04a9, 0x30b2},
+	{"Canon:Powershot G6 (PTP mode)",       0x04a9, 0x30b3},
+	{"Canon:Digital IXUS 500 (PTP mode)",   0x04a9, 0x30b4},
+	{"Canon:PowerShot S500 (PTP mode)",     0x04a9, 0x30b4},
+	{"Canon:PowerShot A75 (PTP mode)",      0x04a9, 0x30b5},
+	{"Canon:PowerShot SD110 (PTP mode)",    0x04a9, 0x30b6},
+	{"Canon:Digital IXUS IIs (PTP mode)",   0x04a9, 0x30b6},
+	{"Canon:PowerShot A400 (PTP mode)",     0x04a9, 0x30b7},
+	{"Canon:PowerShot A310 (PTP mode)",     0x04a9, 0x30b8},
+	{"Canon:PowerShot A85 (PTP mode)",      0x04a9, 0x30b9},
+	{"Canon:Digital IXUS 430 (PTP mode)",   0x04a9, 0x30ba},
+ 	{"Canon:PowerShot S410 (PTP mode)",     0x04a9, 0x30ba},
+ 	{"Canon:PowerShot A95 (PTP mode)",      0x04a9, 0x30bb},
+ 	{"Canon:Digital IXUS 40 (PTP mode)",    0x04a9, 0x30bf},
+ 	{"Canon:EOS 1D Mark II (PTP mode)",     0x04a9, 0x30ea},
+ 	{"Canon:EOS 20D (PTP mode)",            0x04a9, 0x30ec},
+
+	/* Konica-Minolta PTP cameras */
+	{"Konica-Minolta:DiMAGE A2 (PTP mode)",        0x132b, 0x0001},
+	{"Konica-Minolta:DiMAGE Z3 (PictBridge mode)", 0x132b, 0x0018},
+
+	/* Fuji PTP cameras */
+	{"Fuji:FinePix S7000 (PictBridge mode)",0x04cb, 0x0142},	
+
+	/* Ricoh Caplio GX */
+	{"Ricoh:Caplio GX (PTP mode)",          0x05ca, 0x0325},	
 
 	/* more coming soon :) */
 	{NULL, 0, 0}
@@ -428,20 +494,20 @@ struct _PTPData {
 typedef struct _PTPData PTPData;
 
 static short
-ptp_read_func (unsigned char *bytes, unsigned int size, void *data)
+ptp_read_func (unsigned char *bytes, unsigned int size, void *data, unsigned int *readbytes)
 {
 	Camera *camera = ((PTPData *)data)->camera;
 	int result;
 
 	/*
-	 * gp_port_read returns (in case of success) the number of bytes
-	 * read. libptp doesn't need that.
+	 * gp_port_read returns (in case of success) the number of bytes read.
 	 */
 	result = gp_port_read (camera->port, bytes, size);
 	if (result==0) result = gp_port_read (camera->port, bytes, size);
-	if (result >= 0)
+	if (result >= 0) {
+		*readbytes = result;
 		return (PTP_RC_OK);
-	else
+	} else
 	{
 		perror("gp_port_read");
 		return (translate_gp_result (result));
@@ -469,26 +535,28 @@ ptp_write_func (unsigned char *bytes, unsigned int size, void *data)
 }
 
 static short
-ptp_check_int (unsigned char *bytes, unsigned int size, void *data)
+ptp_check_int (unsigned char *bytes, unsigned int size, void *data, unsigned int *rlen)
 {
 	Camera *camera = ((PTPData *)data)->camera;
 	int result;
 
 	/*
 	 * gp_port_check_int returns (in case of success) the number of bytes
-	 * read. libptp doesn't need that.
+	 * read.
 	 */
 
 	result = gp_port_check_int (camera->port, bytes, size);
 	if (result==0) result = gp_port_check_int (camera->port, bytes, size);
-	if (result >= 0)
+	if (result >= 0) {
+		*rlen = result;
 		return (PTP_RC_OK);
-	else
+	} else {
 		return (translate_gp_result (result));
+	}
 }
 
 static short
-ptp_check_int_fast (unsigned char *bytes, unsigned int size, void *data)
+ptp_check_int_fast (unsigned char *bytes, unsigned int size, void *data, unsigned int *rlen)
 {
 	Camera *camera = ((PTPData *)data)->camera;
 	int result;
@@ -500,20 +568,28 @@ ptp_check_int_fast (unsigned char *bytes, unsigned int size, void *data)
 
 	result = gp_port_check_int_fast (camera->port, bytes, size);
 	if (result==0) result = gp_port_check_int_fast (camera->port, bytes, size);
-	if (result >= 0)
+	if (result >= 0) {
+		*rlen = result;
 		return (PTP_RC_OK);
-	else
+	} else {
 		return (translate_gp_result (result));
+	}
 }
 
 
 static void
+#ifdef __GNUC__
+__attribute__((__format__(printf,2,0)))
+#endif
 ptp_debug_func (void *data, const char *format, va_list args)
 {
 	gp_logv (GP_LOG_DEBUG, "ptp", format, args);
 }
 
 static void
+#ifdef __GNUC__
+__attribute__((__format__(printf,2,0)))
+#endif
 ptp_error_func (void *data, const char *format, va_list args)
 {
 	PTPData *ptp_data = data;
@@ -580,18 +656,22 @@ camera_exit (Camera *camera, GPContext *context)
 
 	/* get port settings */
 	CR (gp_port_get_settings (camera->port, &settings));
-	if (camera->pl) {
-#if 0
-		/* it won't hurt */
-		GP_DEBUG ("Clearing STALL condition on ep: 0x%x",
-		settings.usb.outep);
-		gp_port_usb_clear_halt(camera->port, settings.usb.outep);
-#endif
+	if (camera->pl!=NULL) {
 		/* close ptp session */
 		ptp_closesession (&camera->pl->params);
 		free (camera->pl);
 		camera->pl = NULL;
 	}
+	if (camera->port!=NULL)  {
+		/* clear halt */
+		gp_port_usb_clear_halt
+				(camera->port, GP_PORT_USB_ENDPOINT_IN);
+		gp_port_usb_clear_halt
+				(camera->port, GP_PORT_USB_ENDPOINT_OUT);
+		gp_port_usb_clear_halt
+				(camera->port, GP_PORT_USB_ENDPOINT_INT);
+	}
+
 	/* FIXME: free all camera->pl->params.objectinfo[] and
 	   other malloced data */
 
@@ -646,8 +726,11 @@ camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
 	}
 
 	if (!ptp_operation_issupported(&camera->pl->params,
-		PTP_OC_InitiateCapture))
+		PTP_OC_InitiateCapture)) {
+		gp_context_error(context,
+               	_("Sorry, your camera does not support generic capture"));
 		return GP_ERROR_NOT_SUPPORTED;
+	}
 
 	/* A capture may take longer than the standard 8 seconds.
 	 * The G5 for instance does, or in dark rooms ...
@@ -659,27 +742,139 @@ camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
 	 */
 	CPR(context,ptp_initiatecapture(&camera->pl->params, 0x00000000, 0x00000000));
 	CR (gp_port_set_timeout (camera->port, USB_TIMEOUT_CAPTURE));
-	if (ptp_usb_event_wait(&camera->pl->params,&event)!=PTP_RC_OK) goto out;
-	if (event.Code==PTP_EC_ObjectAdded) {
-		if (ptp_usb_event_wait(&camera->pl->params, &event)!=PTP_RC_OK)
+	/* A word of comments is worth here.
+	 * After InitiateCapture camera should report with ObjectAdded event
+	 * all newly created objects. However there might be more than one
+	 * newly created object. There a two scenarios here, which may occur
+	 * both at the time.
+	 * 1) InitiateCapture trigers capture of more than one object if the
+	 * camera is in burst mode for example.
+	 * 2) InitiateCapture creates a number of objects, but not all
+	 * objects represents images. This happens when the camera creates a
+	 * folder for newly captured image(s). This may happen with the
+	 * fresh, formatted flashcard or in burs mode if the camera is
+	 * configured to create a dedicated folder for a burst of pictures.
+	 * The newly created folder (an association object) is reported
+	 * before the images that are stored after its creation.
+	 * Thus we set CameraFilePath to the path to last object reported by
+	 * the camera.
+	 */
+	{
+		short ret;
+		ret=ptp_usb_event_wait(&camera->pl->params,&event);
+		CR (gp_port_set_timeout (camera->port, USB_TIMEOUT));
+		if (ret!=PTP_RC_OK) goto err;
+	}
+	while (event.Code==PTP_EC_ObjectAdded) {
+		/* add newly created object to internal structures */
+		add_object (camera, event.Param1, context);
+
+		/* I hate workarounds! Nikon is not 100% PTP compatible here! */
+		if (camera->pl->params.deviceinfo.VendorExtensionID==PTP_VENDOR_NIKON) 
 			goto out;
-		if (event.Code==PTP_EC_CaptureComplete) {
-			return GP_OK;
-			CR (gp_port_set_timeout (camera->port, USB_TIMEOUT));
+		if (ptp_usb_event_wait(&camera->pl->params, &event)!=PTP_RC_OK)
+		{
+			gp_context_error (context,
+			_("Capture command completed, but no confirmation received"));
+			goto err;
 		}
 	} 
+	if (event.Code==PTP_EC_CaptureComplete) 
+		goto out;
+
+	gp_context_error (context,_("Received event 0x%04x"),event.Code);
+
+	err:
+	/* we're not setting *path on error! */
+	return GP_ERROR;
 
 	out:
-	CR (gp_port_set_timeout (camera->port, USB_TIMEOUT));
+	/* we gonna set the *path now... */
 
-	/* we're not going to set path, ptp does not use paths anyway ;) */
-	return GP_ERROR;
+	/*
+        CR (gp_filesystem_append (camera->fs, path->folder,
+		path->name, context));
+
+	*/
+
+	return GP_OK;
+
+}
+
+static void
+_value_to_str(PTPPropertyValue *data, uint16_t dt, char *txt) {
+	if (dt == PTP_DTC_STR) {
+		sprintf (txt, "'%s'", data->str);
+		return;
+	}
+	if (dt & PTP_DTC_ARRAY_MASK) {
+		int i;
+
+		sprintf (txt, "a[%d] ", data->a.count);
+		txt += strlen(txt);
+		for ( i=0; i<data->a.count; i++) {
+			_value_to_str(&data->a.v[i], dt & ~PTP_DTC_ARRAY_MASK, txt);
+			txt += strlen(txt);
+			if (i!=data->a.count-1) {
+				sprintf (txt, ",");
+				txt++;
+			}
+		}
+	} else {
+		switch (dt) {
+		case PTP_DTC_UNDEF: 
+			sprintf (txt, "Undefined");
+			break;
+		case PTP_DTC_INT8:
+			sprintf (txt, "%d", data->i8);
+			break;
+		case PTP_DTC_UINT8:
+			sprintf (txt, "%u", data->u8);
+			break; 
+		case PTP_DTC_INT16:
+			sprintf (txt, "%d", data->i16);
+			break;
+		case PTP_DTC_UINT16:
+			sprintf (txt, "%u", data->u16);
+			break;
+		case PTP_DTC_INT32:
+			sprintf (txt, "%d", data->i32);
+			break;
+		case PTP_DTC_UINT32:
+			sprintf (txt, "%u", data->u32);
+			break;
+	/*
+		PTP_DTC_INT64           
+		PTP_DTC_UINT64         
+		PTP_DTC_INT128        
+		PTP_DTC_UINT128      
+	*/
+		default:
+			sprintf (txt, "Unknown %x", dt);
+			break;
+		}
+	}
+	return;
+}
+
+static const char *
+_get_getset(uint8_t gs) {
+	switch (gs) {
+	case PTP_DPGS_Get: return N_("read only");
+	case PTP_DPGS_GetSet: return N_("readwrite");
+	default: return N_("Unknown");
+	}
+	return N_("Unknown");
 }
 
 static int
 camera_summary (Camera* camera, CameraText* summary, GPContext *context)
 {
-	int n;
+	int n, i, j;
+	char *txt;
+	PTPParams *params = &(camera->pl->params);
+	PTPDeviceInfo pdi;
+
 	n = snprintf (summary->text, sizeof (summary->text),
 		_("Model: %s\n"
 		"  device version: %s\n"
@@ -692,79 +887,554 @@ camera_summary (Camera* camera, CameraText* summary, GPContext *context)
 		camera->pl->params.deviceinfo.SerialNumber,
 		camera->pl->params.deviceinfo.VendorExtensionID,
 		camera->pl->params.deviceinfo.VendorExtensionDesc);
-	if (n>=sizeof(summary->text))
+
+	if (n>=sizeof (summary->text))
 		return GP_OK;
 
-	
+	txt = summary->text + strlen (summary->text);
+
+	/* The information is cached. However, the canon firmware changes
+	 * the available properties in capture mode.
+	 */
+	CPR(context, ptp_getdeviceinfo(&camera->pl->params, &pdi));
+
+        for (i=0;i<pdi.DevicePropertiesSupported_len;i++) {
+		PTPDevicePropDesc dpd;
+		unsigned int dpc = pdi.DevicePropertiesSupported[i];
+		const char *propname = ptp_get_property_description (params, dpc);
+
+		if (propname) {
+			/* string registered for i18n in ptp.c. */
+			sprintf(txt, "%s(0x%04x):", _(propname), dpc);
+		} else {
+			sprintf(txt, "Property 0x%04x:", dpc);
+		}
+		txt = txt+strlen(txt);
+		
+		memset (&dpd, 0, sizeof (dpd));
+		ptp_getdevicepropdesc (params, dpc, &dpd);
+
+		sprintf (txt, "(%s) ",_get_getset(dpd.GetSet));txt += strlen (txt);
+		sprintf (txt, "(type=0x%x) ",dpd.DataType);txt += strlen (txt);
+		switch (dpd.FormFlag) {
+		case PTP_DPFF_None:	break;
+		case PTP_DPFF_Range: {
+			sprintf (txt, "Range [");txt += strlen(txt);
+			_value_to_str (&dpd.FORM.Range.MinimumValue, dpd.DataType, txt);
+			txt += strlen(txt);
+			sprintf (txt, " - ");txt += strlen(txt);
+			_value_to_str (&dpd.FORM.Range.MaximumValue, dpd.DataType, txt);
+			txt += strlen(txt);
+			sprintf (txt, ", step ");txt += strlen(txt);
+			_value_to_str (&dpd.FORM.Range.StepSize, dpd.DataType, txt);
+			txt += strlen(txt);
+			sprintf (txt, "] value: ");txt += strlen(txt);
+			txt += strlen(txt);
+			break;
+		}
+		case PTP_DPFF_Enumeration:
+			sprintf (txt, "Enumeration [");txt += strlen(txt);
+			if ((dpd.DataType & PTP_DTC_ARRAY_MASK) == PTP_DTC_ARRAY_MASK)  {
+				sprintf (txt, "\n\t");txt += strlen(txt);
+			}
+			for (j = 0; j<dpd.FORM.Enum.NumberOfValues; j++) {
+				_value_to_str(dpd.FORM.Enum.SupportedValue+j,dpd.DataType,txt);
+				txt += strlen(txt);
+				if (j != dpd.FORM.Enum.NumberOfValues-1) {
+					sprintf (txt, ",");txt += strlen(txt);
+					if ((dpd.DataType & PTP_DTC_ARRAY_MASK) == PTP_DTC_ARRAY_MASK)  {
+						sprintf (txt, "\n\t");txt += strlen(txt);
+					}
+				}
+			}
+			if ((dpd.DataType & PTP_DTC_ARRAY_MASK) == PTP_DTC_ARRAY_MASK)  {
+				sprintf (txt, "\n\t");txt += strlen(txt);
+			}
+			sprintf (txt, "] value: ");txt += strlen(txt);
+			break;
+		}
+		txt += strlen (txt);
+		_value_to_str (&dpd.CurrentValue, dpd.DataType, txt);
+		txt += strlen(txt);
+		sprintf(txt,"\n");
+		txt += strlen(txt);
+        }
 	return (GP_OK);
 }
 
+static int
+have_prop(Camera *camera, uint16_t vendor, uint16_t prop) {
+	int i;
+
+	for (i=0; i<camera->pl->params.deviceinfo.DevicePropertiesSupported_len; i++) {
+		if (prop != camera->pl->params.deviceinfo.DevicePropertiesSupported[i])
+			continue;
+		if ((prop & 0xf000) == 0x5000) /* generic property */
+			return 1;
+		if (camera->pl->params.deviceinfo.VendorExtensionID==vendor)
+			return 1;
+	}
+	return 0;
+}
+
+struct submenu;
+typedef int (*get_func)(CameraWidget **widget, struct submenu* menu, PTPDevicePropDesc *dpd);
+typedef int (*put_func)(CameraWidget *widget, PTPPropertyValue *propval);
+
+struct submenu {
+	char 		*label;
+	char		*name;
+	uint16_t	propid;
+	uint16_t	vendorid;
+	uint16_t	type;
+	get_func	getfunc;
+	put_func	putfunc;
+};
+
+struct menu {
+	char		*label;
+	char		*name;
+	struct	submenu	*submenus;
+};
+
+static int
+_get_AUINT8_as_CHAR_ARRAY(CameraWidget **widget, struct submenu *menu, PTPDevicePropDesc *dpd) {
+	int	j;
+	char 	value[128];
+
+	gp_widget_new (GP_WIDGET_TEXT, _(menu->label), widget);
+	gp_widget_set_name (*widget, menu->name);
+	if (dpd->DataType != PTP_DTC_AUINT8) {
+		sprintf (value,_("unexpected datatype %i"),dpd->DataType);
+	} else {
+		memset(value,0,sizeof(value));
+		for (j=0;j<dpd->CurrentValue.a.count;j++)
+			value[j] = dpd->CurrentValue.a.v[j].u8;
+	}
+	gp_widget_set_value (*widget,value);
+	return (GP_OK);
+}
+
+static
+int _get_STR(CameraWidget **widget, struct submenu *menu, PTPDevicePropDesc *dpd) {
+	char value[64];
+
+	gp_widget_new (GP_WIDGET_TEXT, _(menu->label), widget);
+	gp_widget_set_name (*widget, menu->name);
+	if (dpd->DataType != PTP_DTC_STR) {
+		sprintf (value,_("unexpected datatype %i"),dpd->DataType);
+		gp_widget_set_value (*widget,value);
+	} else {
+		gp_widget_set_value (*widget,dpd->CurrentValue.str);
+	}
+	return (GP_OK);
+}
+
+
+static int
+_put_AUINT8_as_CHAR_ARRAY(CameraWidget *widget, PTPPropertyValue *propval) {
+	char	*value;
+	int	i, ret;
+
+	ret = gp_widget_get_value (widget, &value);
+	if (ret != GP_OK)
+		return ret;
+	fprintf (stderr, "value is %s\n", value);
+	memset(propval,0,sizeof(PTPPropertyValue));
+	/* add \0 ? */
+	propval->a.v = malloc(strlen(value)*sizeof(PTPPropertyValue));
+	propval->a.count = strlen(value);
+	for (i=0;i<strlen(value);i++)
+		propval->a.v[i].u8 = value[i];
+	return (GP_OK);
+}
+
+static int
+_get_UINT32_as_MB(CameraWidget **widget, struct submenu *menu, PTPDevicePropDesc *dpd) {
+	char value[64];
+
+	gp_widget_new (GP_WIDGET_TEXT, _(menu->label), widget);
+	gp_widget_set_name (*widget, menu->name);
+	if (dpd->DataType != PTP_DTC_UINT32) {
+		sprintf (value,_("unexpected datatype %i"),dpd->DataType);
+	} else {
+		sprintf (value,"%i",dpd->CurrentValue.u32/1024/1024);
+	}
+	gp_widget_set_value (*widget,value);
+	return (GP_OK);
+}
+
+static int
+_get_Canon_AssistLight(CameraWidget **widget, struct submenu *menu, PTPDevicePropDesc *dpd) {
+	gp_widget_new (GP_WIDGET_RADIO, _(menu->label), widget);
+	gp_widget_set_name (*widget, menu->name);
+	if (!(dpd->FormFlag & PTP_DPFF_Enumeration))
+		return (GP_ERROR);
+	if (dpd->DataType != PTP_DTC_UINT16)
+		return (GP_ERROR);
+	gp_widget_add_choice (*widget,_("On"));
+	gp_widget_add_choice (*widget,_("Off"));
+	gp_widget_set_value (*widget,(dpd->CurrentValue.u16?_("On"):_("Off")));
+	return (GP_OK);
+}
+
+static int
+_put_Canon_AssistLight(CameraWidget *widget, PTPPropertyValue *propval) {
+	char *value;
+	int ret;
+
+	ret = gp_widget_get_value (widget, &value);
+	if (ret != GP_OK)
+		return ret;
+	if (!strcmp (value, _("On"))) {
+		propval->u16 = 1;
+		return (GP_OK);
+	}
+	if (!strcmp (value, _("Off"))) {
+		propval->u16 = 0;
+		return (GP_OK);
+	}
+	return (GP_ERROR);
+}
+
+static int
+_get_Canon_ExpCompensation(CameraWidget **widget, struct submenu *menu, PTPDevicePropDesc *dpd) {
+	float t, b, s, f;
+	int min, max, i;
+
+	min = 256; max = 0;
+	gp_widget_new (GP_WIDGET_RANGE, _(menu->label), widget);
+	gp_widget_set_name (*widget, menu->name);
+	if (!(dpd->FormFlag & PTP_DPFF_Enumeration))
+		return (GP_ERROR);
+	if (dpd->DataType != PTP_DTC_UINT8)
+		return (GP_ERROR);
+	f = (float)dpd->CurrentValue.u8;
+
+	for (i=0; i<dpd->FORM.Enum.NumberOfValues;i++) {
+		if (min > dpd->FORM.Enum.SupportedValue[i].u8)
+			min = dpd->FORM.Enum.SupportedValue[i].u8;
+		if (max < dpd->FORM.Enum.SupportedValue[i].u8)
+			max = dpd->FORM.Enum.SupportedValue[i].u8;
+	}
+	/* -2.0  ... 2.0 is shown on the camera, the property has:
+	   Belichtungskompensation(0xd01f):(readwrite) (type=0x2) Enumeration [8,11,13,16,19,21,24,27,29,32,35,37,40] value: 24
+	 */
+	f = f * 20./((float)(max - min));
+	b =   0.0;
+	t =  20.0;
+	s =   1.0; /* actually just 13 steps on my canon */
+	gp_widget_set_range (*widget, b, t, s);
+	gp_widget_set_value (*widget, &f);
+	return (GP_OK);
+}
+
+static int
+_put_Canon_ExpCompensation(CameraWidget *widget, PTPPropertyValue *propval) {
+	float value;
+	int ret;
+
+	ret = gp_widget_get_value (widget, &value);
+	if (ret != GP_OK)
+		return ret;
+	fprintf (stderr, "set exp comp to %g\n", value);
+	return (GP_ERROR);
+}
+static int
+_get_Canon_BeepMode(CameraWidget **widget, struct submenu *menu, PTPDevicePropDesc *dpd) {
+	gp_widget_new (GP_WIDGET_RADIO, _(menu->label), widget);
+	gp_widget_set_name (*widget, menu->name);
+	if (!(dpd->FormFlag & PTP_DPFF_Enumeration))
+		return (GP_ERROR);
+	if (dpd->DataType != PTP_DTC_UINT8)
+		return (GP_ERROR);
+	gp_widget_add_choice (*widget,_("On"));
+	gp_widget_add_choice (*widget,_("Off"));
+	gp_widget_set_value (*widget,(dpd->CurrentValue.u8?_("On"):_("Off")));
+	return (GP_OK);
+}
+
+static int
+_put_Canon_BeepMode(CameraWidget *widget, PTPPropertyValue *propval) {
+	char *value;
+	int ret;
+
+	ret = gp_widget_get_value (widget, &value);
+	if (ret != GP_OK)
+		return ret;
+	if (!strcmp (value, _("On"))) {
+		propval->u8 = 1;
+		return (GP_OK);
+	}
+	if (!strcmp (value, _("Off"))) {
+		propval->u8 = 0;
+		return (GP_OK);
+	}
+	return (GP_ERROR);
+}
+
+static int
+_get_Canon_ZoomRange(CameraWidget **widget, struct submenu *menu, PTPDevicePropDesc *dpd) {
+	float	f, t, b, s;
+
+	gp_widget_new (GP_WIDGET_RANGE, _(menu->label), widget);
+	gp_widget_set_name (*widget,menu->name);
+	f = (float)dpd->CurrentValue.u16;
+	if (!(dpd->FormFlag & PTP_DPFF_Range))
+		return (GP_ERROR);
+	b = (float)dpd->FORM.Range.MinimumValue.u16;
+	t = (float)dpd->FORM.Range.MaximumValue.u16;
+	s = (float)dpd->FORM.Range.StepSize.u16;
+	gp_widget_set_range (*widget, b, t, s);
+	gp_widget_set_value (*widget, &f);
+	return (GP_OK);
+}
+
+static int
+_put_Canon_ZoomRange(CameraWidget *widget, PTPPropertyValue *propval)
+{
+	float	f;
+	int	ret;
+
+	f = 0.0;
+	ret = gp_widget_get_value (widget,&f);
+	if (ret != GP_OK) return ret;
+	propval->u16 = (unsigned short)f;
+	return (GP_OK);
+}
+
+static int
+_get_UINT32_as_time(CameraWidget **widget, struct submenu *menu, PTPDevicePropDesc *dpd) {
+	time_t	camtime;
+
+	gp_widget_new (GP_WIDGET_DATE, _(menu->label), widget);
+	gp_widget_set_name (*widget,menu->name);
+	camtime = dpd->CurrentValue.u32;
+	gp_widget_set_value (*widget,&camtime);
+	return (GP_OK);
+}
+
+static int
+_put_UINT32_as_time(CameraWidget *widget, PTPPropertyValue *propval) {
+	time_t	camtime;
+	int	ret;
+
+	camtime = 0;
+	ret = gp_widget_get_value (widget,&camtime);
+	if (ret != GP_OK)
+		return ret;
+	propval->u32 = camtime;
+	return (GP_OK);
+}
+
+static int
+_get_STR_as_time(CameraWidget **widget, struct submenu *menu, PTPDevicePropDesc *dpd) {
+	time_t		camtime;
+	struct tm	tm;
+	char		capture_date[64],tmp[5];
+
+	/* strptime() is not widely accepted enough to use yet */
+	memset(&tm,0,sizeof(tm));
+	gp_widget_new (GP_WIDGET_DATE, _(menu->label), widget);
+	gp_widget_set_name (*widget, menu->name);
+	strncpy(capture_date,dpd->CurrentValue.str,sizeof(capture_date));
+	strncpy (tmp, capture_date, 4);
+	tmp[4] = 0;
+	tm.tm_year=atoi (tmp) - 1900;
+	strncpy (tmp, capture_date + 4, 2);
+	tmp[2] = 0;
+	tm.tm_mon = atoi (tmp) - 1;
+	strncpy (tmp, capture_date + 6, 2);
+	tmp[2] = 0;
+	tm.tm_mday = atoi (tmp);
+	strncpy (tmp, capture_date + 9, 2);
+	tmp[2] = 0;
+	tm.tm_hour = atoi (tmp);
+	strncpy (tmp, capture_date + 11, 2);
+	tmp[2] = 0;
+	tm.tm_min = atoi (tmp);
+	strncpy (tmp, capture_date + 13, 2);
+	tmp[2] = 0;
+	tm.tm_sec = atoi (tmp);
+	camtime = mktime(&tm);
+	gp_widget_set_value (*widget,&camtime);
+	return (GP_OK);
+}
+
+static int
+_put_STR_as_time(CameraWidget *widget, PTPPropertyValue *propval) {
+	time_t		camtime;
+#ifdef HAVE_GMTIME_R
+	struct tm	xtm;
+#endif
+	struct tm	*pxtm;
+	int		ret;
+	char		asctime[64];
+
+	camtime = 0;
+	ret = gp_widget_get_value (widget,&camtime);
+	if (ret != GP_OK)
+		return ret;
+#ifdef HAVE_GMTIME_R
+	pxtm = gmtime_r (&camtime, &xtm);
+#else
+	pxtm = gmtime (&camtime);
+#endif
+	/* 20020101T123400.0 is returned by the HP Photosmart */
+	sprintf(asctime,"%04d%02d%02dT%02d%02d%02d.0",pxtm->tm_year+1900,pxtm->tm_mon+1,pxtm->tm_mday,pxtm->tm_hour,pxtm->tm_min,pxtm->tm_sec);
+	propval->str = strdup(asctime);
+	return (GP_OK);
+}
+
+static int
+_put_None(CameraWidget *widget, PTPPropertyValue *dpd) {
+	return (GP_ERROR_NOT_SUPPORTED);
+}
+
+struct submenu camera_settings_menu[] = {
+	{ N_("Camera Owner"), "owner", PTP_DPC_CANON_CameraOwner, PTP_VENDOR_CANON, PTP_DTC_AUINT8, _get_AUINT8_as_CHAR_ARRAY, _put_AUINT8_as_CHAR_ARRAY },
+	{ N_("Camera Model"), "model", PTP_DPC_CANON_CameraModel, PTP_VENDOR_CANON, PTP_DTC_STR, _get_STR, _put_None },
+	{ N_("Flash Memory"), "internalmemory", PTP_DPC_CANON_FlashMemory, PTP_VENDOR_CANON, PTP_DTC_UINT32, _get_UINT32_as_MB, _put_None },
+	{ N_("Camera Time"),  "time", PTP_DPC_CANON_UnixTime,     PTP_VENDOR_CANON, PTP_DTC_UINT32, _get_UINT32_as_time, _put_UINT32_as_time },
+	{ N_("Camera Time"),  "time", PTP_DPC_DateTime,           0,                PTP_DTC_STR, _get_STR_as_time, _put_STR_as_time },
+	{ N_("Beep Mode"),  "beep",   PTP_DPC_CANON_BeepMode,     PTP_VENDOR_CANON, PTP_DTC_UINT8, _get_Canon_BeepMode, _put_Canon_BeepMode },
+	{ NULL },
+};
+
+struct submenu capture_settings_menu[] = {
+	{ N_("Zoom"), "zoom", PTP_DPC_CANON_Zoom, PTP_VENDOR_CANON, PTP_DTC_UINT16, _get_Canon_ZoomRange, _put_Canon_ZoomRange},
+	{ N_("Assist Light"), "assistlight", PTP_DPC_CANON_AssistLight, PTP_VENDOR_CANON, PTP_DTC_UINT16, _get_Canon_AssistLight, _put_Canon_AssistLight},
+	{ N_("Exposure Compensation"), "exposurecompensation", PTP_DPC_CANON_ExpCompensation, PTP_VENDOR_CANON, PTP_DTC_UINT8, _get_Canon_ExpCompensation, _put_Canon_ExpCompensation},
+	{ NULL },
+};
+
+struct menu menus[] = {
+	{ N_("Camera Settings"), "settings", camera_settings_menu },
+	{ N_("Capture Settings"), "capturesettings", capture_settings_menu },
+};
+
 static int
 camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 {
 	CameraWidget *section, *widget;
 	PTPDevicePropDesc dpd;
-	char value[255];
+	int menuno, submenuno;
 
-	memset(&dpd,0,sizeof(dpd));
-	ptp_getdevicepropdesc(&camera->pl->params,PTP_DPC_BatteryLevel,&dpd);
-	GP_DEBUG ("Data Type = 0x%.4x",dpd.DataType);
-	GP_DEBUG ("Get/Set = 0x%.2x",dpd.GetSet);
-	GP_DEBUG ("Form Flag = 0x%.2x",dpd.FormFlag);
-	if (dpd.DataType!=PTP_DTC_UINT8) {
-		ptp_free_devicepropdesc(&dpd);
-		return GP_ERROR_NOT_SUPPORTED;
-	}
-	GP_DEBUG ("Factory Default Value = %0.2x",*(uint8_t *)dpd.FactoryDefaultValue);
-	GP_DEBUG ("Current Value = %0.2x",*(uint8_t *)dpd.CurrentValue);
+	gp_widget_new (GP_WIDGET_WINDOW, _("Camera and Driver Configuration"), window);
+	gp_widget_set_name (*window, "main");
+	for (menuno = 0; menuno < sizeof(menus)/sizeof(menus[0]) ; menuno++ ) {
+		gp_widget_new (GP_WIDGET_SECTION, _(menus[menuno].label), &section);
+		gp_widget_set_name (section, menus[menuno].name);
+		gp_widget_append (*window, section);
 
-	gp_widget_new (GP_WIDGET_WINDOW, _("Camera and Driver Configuration"),
-		window);
-	gp_widget_new (GP_WIDGET_SECTION, _("Power (readonly)"), &section);
-	gp_widget_append (*window, section);
-	switch (dpd.FormFlag) {
-	case PTP_DPFF_Enumeration: {
-		uint16_t i;
-		char tmp[64];
-
-		GP_DEBUG ("Number of values %i",
-			dpd.FORM.Enum.NumberOfValues);
-		gp_widget_new (GP_WIDGET_TEXT, _("Number of values"),&widget);
-		snprintf (value,255,"%i",dpd.FORM.Enum.NumberOfValues);
-		gp_widget_set_value (widget,value);
-		gp_widget_append (section,widget);
-		gp_widget_new (GP_WIDGET_TEXT, _("Supported values"),&widget);
-		value[0]='\0';
-		for (i=0;i<dpd.FORM.Enum.NumberOfValues;i++){
-			snprintf (tmp,6,"|%.3i|",
-			*(uint8_t *)dpd.FORM.Enum.SupportedValue[i]);
-			strncat(value,tmp,6);
+		for (submenuno = 0; menus[menuno].submenus[submenuno].name ; submenuno++ ) {
+			struct submenu *cursub = menus[menuno].submenus+submenuno;
+			if (!have_prop(camera,cursub->vendorid,cursub->propid)) {
+				continue;
+			}
+			memset(&dpd,0,sizeof(dpd));
+			ptp_getdevicepropdesc(&camera->pl->params,cursub->propid,&dpd);
+			cursub->getfunc (&widget, cursub, &dpd);
+			gp_widget_append (section,widget);
+			ptp_free_devicepropdesc(&dpd);
 		}
-		gp_widget_set_value (widget,value);
-		gp_widget_append (section,widget);
-		gp_widget_new (GP_WIDGET_TEXT, _("Current value"),&widget);
-		snprintf (value,255,"%i",*(uint8_t *)dpd.CurrentValue);
-		gp_widget_set_value (widget,value);
-		gp_widget_append (section,widget);
-		break;
 	}
-	case PTP_DPFF_Range: {
-		float value_float;
-                fprintf (stderr,", within range: %d - %d, stepping %d\n", *(uint8_t*)dpd.FORM.Range.MinimumValue, *(uint8_t*)dpd.FORM.Range.MaximumValue, *(uint8_t*)dpd.FORM.Range.StepSize);
-		gp_widget_new (GP_WIDGET_RANGE, _("Power (readonly)"), &widget);
-		gp_widget_append (section,widget);
-		gp_widget_set_range (widget, *(uint8_t*)dpd.FORM.Range.MinimumValue, *(uint8_t*)dpd.FORM.Range.MaximumValue, *(uint8_t*)dpd.FORM.Range.StepSize);
-        	/* this is a write only capability */
-        	value_float = *(uint8_t *)dpd.CurrentValue;
-        	gp_widget_set_value (widget, &value_float);
-        	gp_widget_changed(widget);
-                break;
+#if 0
+	char value[255];
+	if (have_prop(camera,0,PTP_DPC_BatteryLevel)) {
+		memset(&dpd,0,sizeof(dpd));
+		ptp_getdevicepropdesc(&camera->pl->params,PTP_DPC_BatteryLevel,&dpd);
+		GP_DEBUG ("Data Type = 0x%04x",dpd.DataType);
+		GP_DEBUG ("Get/Set = 0x%02x",dpd.GetSet);
+		GP_DEBUG ("Form Flag = 0x%02x",dpd.FormFlag);
+		if (dpd.DataType!=PTP_DTC_UINT8) {
+			ptp_free_devicepropdesc(&dpd);
+			return GP_ERROR_NOT_SUPPORTED;
+		}
+		GP_DEBUG ("Factory Default Value = %0.2x",dpd.FactoryDefaultValue.u8);
+		GP_DEBUG ("Current Value = %0.2x",dpd.CurrentValue.u8);
+
+		gp_widget_new (GP_WIDGET_SECTION, _("Power (readonly)"), &section);
+		gp_widget_append (*window, section);
+		switch (dpd.FormFlag) {
+		case PTP_DPFF_Enumeration: {
+			uint16_t i;
+			char tmp[64];
+
+			GP_DEBUG ("Number of values %i",
+				dpd.FORM.Enum.NumberOfValues);
+			gp_widget_new (GP_WIDGET_TEXT, _("Number of values"),&widget);
+			snprintf (value,255,"%i",dpd.FORM.Enum.NumberOfValues);
+			gp_widget_set_value (widget,value);
+			gp_widget_append (section,widget);
+			gp_widget_new (GP_WIDGET_TEXT, _("Supported values"),&widget);
+			value[0]='\0';
+			for (i=0;i<dpd.FORM.Enum.NumberOfValues;i++){
+				snprintf (tmp,6,"|%.3i|",dpd.FORM.Enum.SupportedValue[i].u8);
+				strncat(value,tmp,6);
+			}
+			gp_widget_set_value (widget,value);
+			gp_widget_append (section,widget);
+			gp_widget_new (GP_WIDGET_TEXT, _("Current value"),&widget);
+			snprintf (value,255,"%i",dpd.CurrentValue.u8);
+			gp_widget_set_value (widget,value);
+			gp_widget_append (section,widget);
+			break;
+		}
+		case PTP_DPFF_Range: {
+			float value_float;
+			fprintf (stderr,", within range: %d - %d, stepping %d\n", dpd.FORM.Range.MinimumValue.u8, dpd.FORM.Range.MaximumValue.u8, dpd.FORM.Range.StepSize.u8);
+			gp_widget_new (GP_WIDGET_RANGE, _("Power (readonly)"), &widget);
+			gp_widget_append (section,widget);
+			gp_widget_set_range (widget, dpd.FORM.Range.MinimumValue.u8, dpd.FORM.Range.MaximumValue.u8, dpd.FORM.Range.StepSize.u8);
+			/* this is a write only capability */
+			value_float = dpd.CurrentValue.u8;
+			gp_widget_set_value (widget, &value_float);
+			gp_widget_changed(widget);
+			break;
+		}
+		case PTP_DPFF_None:
+			break;
+		default: break;
+		}
 	}
-        case PTP_DPFF_None:
-		break;
-	default: break;
+#endif
+	return GP_OK;
+}
+
+static int
+camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
+{
+	CameraWidget *section, *widget, *subwindow;
+	int menuno, submenuno, ret;
+
+	ret = gp_widget_get_child_by_label (window, _("Camera and Driver Configuration"), &subwindow);
+	if (ret != GP_OK)
+		return ret;
+	for (menuno = 0; menuno < sizeof(menus)/sizeof(menus[0]) ; menuno++ ) {
+		ret = gp_widget_get_child_by_label (subwindow, _(menus[menuno].label), &section);
+		if (ret != GP_OK)
+			continue;
+		for (submenuno = 0; menus[menuno].submenus[submenuno].label ; submenuno++ ) {
+			PTPPropertyValue	propval;
+
+			struct submenu *cursub = menus[menuno].submenus+submenuno;
+			if (!have_prop(camera,cursub->vendorid,cursub->propid))
+				continue;
+
+			ret = gp_widget_get_child_by_label (section, _(cursub->label), &widget);
+			if (ret != GP_OK)
+				continue;
+
+			if (!gp_widget_changed (widget))
+				continue;
+
+			ret = cursub->putfunc (widget, &propval);
+			if (ret == GP_OK)
+				ptp_setdevicepropvalue (&camera->pl->params, cursub->propid, &propval, cursub->type);
+			ptp_free_devicepropvalue (cursub->type, &propval);
+		}
 	}
-	ptp_free_devicepropdesc(&dpd);
 	return GP_OK;
 }
 
@@ -989,9 +1659,11 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		/* we do not allow downloading unknown type files as in most
 		cases they are special file (like firmware or control) which
 		sometimes _cannot_ be downloaded. doing so we avoid errors.*/
-		if ((oi->ObjectFormat == PTP_OFC_Undefined) ||
-			(oi->ObjectFormat == PTP_OFC_Association))
+		if (oi->ObjectFormat == PTP_OFC_Association ||
+			(oi->ObjectFormat == PTP_OFC_Undefined &&
+				oi->ThumbFormat == PTP_OFC_Undefined))
 			return (GP_ERROR_NOT_SUPPORTED);
+
 		size=oi->ObjectCompressedSize;
 		CPR (context, ptp_getobject(&camera->pl->params,
 			camera->pl->params.handles.Handler[object_id],
@@ -1297,8 +1969,7 @@ init_ptp_fs (Camera *camera, GPContext *context)
 int
 camera_init (Camera *camera, GPContext *context)
 {
-	/*GPPortSettings settings;*/
-	short ret,i;
+	int ret, i, retried = 0;
 
 	/* Make sure our port is a USB port */
 	if (camera->port->type != GP_PORT_USB) {
@@ -1312,6 +1983,7 @@ camera_init (Camera *camera, GPContext *context)
 	camera->functions->capture = camera_capture;
 	camera->functions->summary = camera_summary;
 	camera->functions->get_config = camera_get_config;
+	camera->functions->set_config = camera_set_config;
 
 	/* We need some data that we pass around */
 	camera->pl = malloc (sizeof (CameraPrivateLibrary));
@@ -1337,27 +2009,30 @@ camera_init (Camera *camera, GPContext *context)
 	 */
 	CR (gp_port_set_timeout (camera->port, USB_TIMEOUT));
 	/* do we configure port ???*/
-#if 0	
-	/* Configure the port */
-	CR (gp_port_get_settings (camera->port, &settings));
 
-	/* Use the defaults the core parsed */
-	CR (gp_port_set_settings (camera->port, settings));
-#endif
 	/* Establish a connection to the camera */
 	((PTPData *) camera->pl->params.data)->context = context;
-	ret=ptp_opensession (&camera->pl->params, 1);
-	while (ret==PTP_RC_InvalidTransactionID) {
-		camera->pl->params.transaction_id+=10;
+
+	retried = 0;
+	while (1) {
 		ret=ptp_opensession (&camera->pl->params, 1);
-	}
-	if (ret!=PTP_RC_SessionAlreadyOpened && ret!=PTP_RC_OK) {
-		report_result(context, ret);
-		return (translate_ptp_result(ret));
+		while (ret==PTP_RC_InvalidTransactionID) {
+			camera->pl->params.transaction_id+=10;
+			ret=ptp_opensession (&camera->pl->params, 1);
+		}
+		if (ret!=PTP_RC_SessionAlreadyOpened && ret!=PTP_RC_OK) {
+			if (retried < 2) { /* try again */
+				retried++;
+				continue;
+			}
+			report_result(context, ret);
+			return (translate_ptp_result(ret));
+		}
+		break;
 	}
 
 	/* Seems HP does not like getdevinfo outside of session 
-	   although it's legel to do so */
+	   although it's legal to do so */
 	/* get device info */
 	CPR(context, ptp_getdeviceinfo(&camera->pl->params,
 	&camera->pl->params.deviceinfo));
@@ -1371,16 +2046,16 @@ camera_init (Camera *camera, GPContext *context)
 	GP_DEBUG ("Vendor extension description: %s",camera->pl->params.deviceinfo.VendorExtensionDesc);
 	GP_DEBUG ("Supported operations:");
 	for (i=0; i<camera->pl->params.deviceinfo.OperationsSupported_len; i++)
-		GP_DEBUG ("  0x%.4x",
+		GP_DEBUG ("  0x%04x",
 			camera->pl->params.deviceinfo.OperationsSupported[i]);
 	GP_DEBUG ("Events Supported:");
 	for (i=0; i<camera->pl->params.deviceinfo.EventsSupported_len; i++)
-		GP_DEBUG ("  0x%.4x",
+		GP_DEBUG ("  0x%04x",
 			camera->pl->params.deviceinfo.EventsSupported[i]);
 	GP_DEBUG ("Device Properties Supported:");
 	for (i=0; i<camera->pl->params.deviceinfo.DevicePropertiesSupported_len;
 		i++)
-		GP_DEBUG ("  0x%.4x",
+		GP_DEBUG ("  0x%04x",
 			camera->pl->params.deviceinfo.DevicePropertiesSupported[i]);
 
 	/* init internal ptp objectfiles (required for fs implementation) */
