@@ -390,30 +390,47 @@ typedef struct _PTPObjectInfo PTPObjectInfo;
 
 /* Property Describing Dataset, Range Form */
 
+union _PTPPropertyValue {
+	char		*str;	/* malloced */
+	uint8_t		u8;
+	int8_t		i8;
+	uint16_t	u16;
+	int16_t		i16;
+	uint32_t	u32;
+	int32_t		i32;
+	/* XXXX: 64bit and 128 bit signed and unsigned missing */
+	struct array {
+		uint32_t	count;
+		union _PTPPropertyValue	*v;	/* malloced, count elements */
+	} a;
+};
+
+typedef union _PTPPropertyValue PTPPropertyValue;
+
 struct _PTPPropDescRangeForm {
-	void *		MinimumValue;
-	void *		MaximumValue;
-	void *		StepSize;
+	PTPPropertyValue 	MinimumValue;
+	PTPPropertyValue 	MaximumValue;
+	PTPPropertyValue 	StepSize;
 };
 typedef struct _PTPPropDescRangeForm PTPPropDescRangeForm;
 
 /* Property Describing Dataset, Enum Form */
 
 struct _PTPPropDescEnumForm {
-	uint16_t	NumberOfValues;
-	void **		SupportedValue;
+	uint16_t		NumberOfValues;
+	PTPPropertyValue	*SupportedValue;	/* malloced */
 };
 typedef struct _PTPPropDescEnumForm PTPPropDescEnumForm;
 
 /* Device Property Describing Dataset (DevicePropDesc) */
 
 struct _PTPDevicePropDesc {
-	uint16_t	DevicePropertyCode;
-	uint16_t	DataType;
-	uint8_t		GetSet;
-	void *		FactoryDefaultValue;
-	void *		CurrentValue;
-	uint8_t		FormFlag;
+	uint16_t		DevicePropertyCode;
+	uint16_t		DataType;
+	uint8_t			GetSet;
+	PTPPropertyValue	FactoryDefaultValue;
+	PTPPropertyValue	CurrentValue;
+	uint8_t			FormFlag;
 	union	{
 		PTPPropDescEnumForm	Enum;
 		PTPPropDescRangeForm	Range;
@@ -449,16 +466,20 @@ typedef struct _PTPCANONFolderEntry PTPCANONFolderEntry;
 #define PTP_DTC_UINT64		0x0008
 #define PTP_DTC_INT128		0x0009
 #define PTP_DTC_UINT128		0x000A
-#define PTP_DTC_AINT8		0x4001
-#define PTP_DTC_AUINT8		0x4002
-#define PTP_DTC_AINT16		0x4003
-#define PTP_DTC_AUINT16		0x4004
-#define PTP_DTC_AINT32		0x4005
-#define PTP_DTC_AUINT32		0x4006
-#define PTP_DTC_AINT64		0x4007
-#define PTP_DTC_AUINT64		0x4008
-#define PTP_DTC_AINT128		0x4009
-#define PTP_DTC_AUINT128	0x400A
+
+#define PTP_DTC_ARRAY_MASK	0x4000
+
+#define PTP_DTC_AINT8		(PTP_DTC_ARRAY_MASK | PTP_DTC_INT8)
+#define PTP_DTC_AUINT8		(PTP_DTC_ARRAY_MASK | PTP_DTC_UINT8)
+#define PTP_DTC_AINT16		(PTP_DTC_ARRAY_MASK | PTP_DTC_INT16)
+#define PTP_DTC_AUINT16		(PTP_DTC_ARRAY_MASK | PTP_DTC_UINT16)
+#define PTP_DTC_AINT32		(PTP_DTC_ARRAY_MASK | PTP_DTC_INT32)
+#define PTP_DTC_AUINT32		(PTP_DTC_ARRAY_MASK | PTP_DTC_UINT32)
+#define PTP_DTC_AINT64		(PTP_DTC_ARRAY_MASK | PTP_DTC_INT64)
+#define PTP_DTC_AUINT64		(PTP_DTC_ARRAY_MASK | PTP_DTC_UINT64)
+#define PTP_DTC_AINT128		(PTP_DTC_ARRAY_MASK | PTP_DTC_INT128)
+#define PTP_DTC_AUINT128	(PTP_DTC_ARRAY_MASK | PTP_DTC_UINT128)
+
 #define PTP_DTC_STR		0xFFFF
 
 /* Device Properties Codes */
@@ -537,6 +558,90 @@ typedef struct _PTPCANONFolderEntry PTPCANONFolderEntry;
 #define PTP_DPC_CANON_AssistLight	0xD041
 #define PTP_DPC_CANON_D045		0xD045
 
+/* Nikon extension device property codes */
+#define PTP_DPC_NIKON_ShootingBank			0xD010
+#define PTP_DPC_NIKON_ShootingBankNameA 		0xD011
+#define PTP_DPC_NIKON_ShootingBankNameB			0xD012
+#define PTP_DPC_NIKON_ShootingBankNameC			0xD013
+#define PTP_DPC_NIKON_ShootingBankNameD			0xD014
+#define PTP_DPC_NIKON_RawCompression			0xD016
+#define PTP_DPC_NIKON_WhiteBalanceAutoBias		0xD017
+#define PTP_DPC_NIKON_WhiteBalanceTungstenBias		0xD018
+#define PTP_DPC_NIKON_WhiteBalanceFlourescentBias	0xD019
+#define PTP_DPC_NIKON_WhiteBalanceDaylightBias		0xD01a
+#define PTP_DPC_NIKON_WhiteBalanceFlashBias		0xD01b
+#define PTP_DPC_NIKON_WhiteBalanceCloudyBias		0xD01c
+#define PTP_DPC_NIKON_WhiteBalanceShadeBias		0xD01d
+#define PTP_DPC_NIKON_WhiteBalanceColourTemperature	0xD01e
+#define PTP_DPC_NIKON_ImageSharpening			0xD02a
+#define PTP_DPC_NIKON_ToneCompensation			0xD02b
+#define PTP_DPC_NIKON_ColourMode			0xD02c
+#define PTP_DPC_NIKON_HueAdjustment			0xD02d
+#define PTP_DPC_NIKON_NonCPULensDataFocalLength		0xD02e
+#define PTP_DPC_NIKON_NonCPULensDataMaximumAperature	0xD02f
+#define PTP_DPC_NIKON_CSMMenuBankSelect			0xD040
+#define PTP_DPC_NIKON_MenuBankNameA			0xD041
+#define PTP_DPC_NIKON_MenuBankNameB			0xD042
+#define PTP_DPC_NIKON_MenuBankNameC			0xD043
+#define PTP_DPC_NIKON_MenuBankNameD			0xD044
+#define PTP_DPC_NIKON_A1AFCModePriority			0xD048
+#define PTP_DPC_NIKON_A2AFSModePriority			0xD049
+#define PTP_DPC_NIKON_A3GroupDynamicAF			0xD04a
+#define PTP_DPC_NIKON_A4AFActivation			0xD04b
+#define PTP_DPC_NIKON_A5FocusAreaIllumManualFocus	0xD04c
+#define PTP_DPC_NIKON_FocusAreaIllumContinuous		0xD04d
+#define PTP_DPC_NIKON_FocusAreaIllumWhenSelected 	0xD04e
+#define PTP_DPC_NIKON_A6FocusArea			0xD04f
+#define PTP_DPC_NIKON_A7VerticalAFON			0xD050
+#define PTP_DPC_NIKON_B1ISOAuto				0xD054
+#define PTP_DPC_NIKON_B2ISOStep				0xD055
+#define PTP_DPC_NIKON_B3EVStep				0xD056
+#define PTP_DPC_NIKON_B4ExposureCompEv			0xD057
+#define PTP_DPC_NIKON_B5ExposureComp			0xD058
+#define PTP_DPC_NIKON_B6CenterWeightArea		0xD059
+#define PTP_DPC_NIKON_C1AELock				0xD05e
+#define PTP_DPC_NIKON_C2AELAFL				0xD05f
+#define PTP_DPC_NIKON_C3AutoMeterOff			0xD062
+#define PTP_DPC_NIKON_C4SelfTimer			0xD063
+#define PTP_DPC_NIKON_C5MonitorOff			0xD064
+#define PTP_DPC_NIKON_D1ShootingSpeed			0xD068
+#define PTP_DPC_NIKON_D2MaximumShots			0xD069
+#define PTP_DPC_NIKON_D3ExpDelayMode			0xD06a
+#define PTP_DPC_NIKON_D4LongExposureNoiseReduction	0xD06b
+#define PTP_DPC_NIKON_D5FileNumberSequence		0xD06c
+#define PTP_DPC_NIKON_D6ControlPanelFinderRearControl	0xD06d
+#define PTP_DPC_NIKON_ControlPanelFinderViewfinder	0xD06e
+#define PTP_DPC_NIKON_D7Illumination			0xD06f
+#define PTP_DPC_NIKON_E1FlashSyncSpeed			0xD074
+#define PTP_DPC_NIKON_E2FlashShutterSpeed		0xD075
+#define PTP_DPC_NIKON_E3AAFlashMode			0xD076
+#define PTP_DPC_NIKON_E4ModelingFlash			0xD077
+#define PTP_DPC_NIKON_E5AutoBracketySet			0xD078
+#define PTP_DPC_NIKON_E6ManualModeBracketing		0xD079	
+#define PTP_DPC_NIKON_E7AutoBracketOrder		0xD07a
+#define PTP_DPC_NIKON_E8AutoBracketSelection		0xD07b
+#define PTP_DPC_NIKON_F1CenterButtonShootingMode	0xD080
+#define PTP_DPC_NIKON_CenterButtonPlaybackMode		0xD081
+#define PTP_DPC_NIKON_F2Multiselector			0xD082
+#define PTP_DPC_NIKON_F3PhotoInfoPlayback		0xD083
+#define PTP_DPC_NIKON_F4AssignFuncButton		0xD084
+#define PTP_DPC_NIKON_F5CustomizeCommDials		0xD085
+#define PTP_DPC_NIKON_ChangeMainSub			0xD086
+#define PTP_DPC_NIKON_AperatureSetting			0xD087
+#define PTP_DPC_NIKON_MenusAndPlayback			0xD088
+#define PTP_DPC_NIKON_F6ButtonsAndDials			0xD089
+#define PTP_DPC_NIKON_F7NoCFCard			0xD08a
+#define PTP_DPC_NIKON_AutoImageRotation			0xD092
+#define PTP_DPC_NIKON_ExposureBracketingOnOff		0xD0c0
+#define PTP_DPC_NIKON_ExposureBracketingIntervalDist	0xD0c1
+#define PTP_DPC_NIKON_ExposureBracketingNumBracketPlace	0xD0c2
+#define PTP_DPC_NIKON_AutofocusLCDTopMode2		0xD107
+#define PTP_DPC_NIKON_AutofocusLCDTopMode3AndMode4	0xD108
+#define PTP_DPC_NIKON_LightMeter			0xD10a
+#define PTP_DPC_NIKON_ExposureAperatureLock(ReadOnly)	0xD111
+#define PTP_DPC_NIKON_MaximumShots			0xD103
+
+
 /* Device Property Form Flag */
 
 #define PTP_DPFF_None			0x00
@@ -553,7 +658,7 @@ typedef struct _PTPParams PTPParams;
 
 /* raw write functions */
 typedef short (* PTPIOReadFunc)	(unsigned char *bytes, unsigned int size,
-				 void *data);
+				 void *data, unsigned int *readlen);
 typedef short (* PTPIOWriteFunc)(unsigned char *bytes, unsigned int size,
 				 void *data);
 /*
@@ -565,10 +670,18 @@ typedef uint16_t (* PTPIOSendData)	(PTPParams* params, PTPContainer* ptp,
 					unsigned char *data, unsigned int size);
 typedef uint16_t (* PTPIOGetResp)	(PTPParams* params, PTPContainer* resp);
 typedef uint16_t (* PTPIOGetData)	(PTPParams* params, PTPContainer* ptp,
-					unsigned char **data);
+					unsigned char **data, unsigned int *recvlen);
 /* debug functions */
-typedef void (* PTPErrorFunc) (void *data, const char *format, va_list args);
-typedef void (* PTPDebugFunc) (void *data, const char *format, va_list args);
+typedef void (* PTPErrorFunc) (void *data, const char *format, va_list args)
+#if (__GNUC__ >= 3)
+	__attribute__((__format__(printf,2,0)))
+#endif
+;
+typedef void (* PTPDebugFunc) (void *data, const char *format, va_list args)
+#if (__GNUC__ >= 3)
+	__attribute__((__format__(printf,2,0)))
+#endif
+;
 
 struct _PTPParams {
 	/* data layer byteorder */
@@ -612,7 +725,7 @@ uint16_t ptp_usb_senddata	(PTPParams* params, PTPContainer* ptp,
 				unsigned char *data, unsigned int size);
 uint16_t ptp_usb_getresp	(PTPParams* params, PTPContainer* resp);
 uint16_t ptp_usb_getdata	(PTPParams* params, PTPContainer* ptp, 
-				unsigned char **data);
+				unsigned char **data, unsigned int *readlen);
 uint16_t ptp_usb_event_check	(PTPParams* params, PTPContainer* event);
 uint16_t ptp_usb_event_wait		(PTPParams* params, PTPContainer* event);
 
@@ -653,9 +766,9 @@ uint16_t ptp_initiatecapture	(PTPParams* params, uint32_t storageid,
 uint16_t ptp_getdevicepropdesc	(PTPParams* params, uint16_t propcode,
 				PTPDevicePropDesc *devicepropertydesc);
 uint16_t ptp_getdevicepropvalue	(PTPParams* params, uint16_t propcode,
-				void** value, uint16_t datatype);
+				PTPPropertyValue* value, uint16_t datatype);
 uint16_t ptp_setdevicepropvalue (PTPParams* params, uint16_t propcode,
-                        	void* value, uint16_t datatype);
+                        	PTPPropertyValue* value, uint16_t datatype);
 
 
 uint16_t ptp_ek_sendfileobjectinfo (PTPParams* params, uint32_t* store,
@@ -701,7 +814,10 @@ int ptp_operation_issupported	(PTPParams* params, uint16_t operation);
 int ptp_property_issupported	(PTPParams* params, uint16_t property);
 
 void ptp_free_devicepropdesc	(PTPDevicePropDesc* dpd);
+void ptp_free_devicepropvalue	(uint16_t dt, PTPPropertyValue* dpd);
 void ptp_perror			(PTPParams* params, uint16_t error);
 
+const char*
+ptp_get_property_description(PTPParams* params, uint16_t dpc);
 
 #endif /* __PTP_H__ */
