@@ -69,10 +69,14 @@ g3_channel_read(GPPort *port, int *channel, char **buffer, int *len)
 
 	*channel = xbuf[1];
 	*len = xbuf[4] + (xbuf[5]<<8) + (xbuf[6]<<16) + (xbuf[7]<<24);
+	/* Safety buffer of 0x800 ... we can only read in 0x800 chunks, 
+	 * otherwise the communication gets hickups. However *len might be
+	 * less.
+	 */
 	if (!*buffer)
-		*buffer = malloc(*len + 1);
+		*buffer = malloc(*len + 1 + 0x800);
 	else
-		*buffer = realloc(*buffer, *len + 1);
+		*buffer = realloc(*buffer, *len + 1 + 0x800);
 	tocopy = *len;
 	if (tocopy > 0x800-8) tocopy = 0x800-8;
 	memcpy(*buffer, xbuf+8, tocopy);
@@ -103,7 +107,7 @@ g3_channel_read_bytes(
 		*buffer = realloc (*buffer, expected);
 	xbuf = malloc(65536 + 12);
 
-	id = gp_context_progress_start (context, expected, msg);
+	id = gp_context_progress_start (context, expected, "%s", msg);
 
 	/* The camera lets us read in packets of at least max 64kb size. */
 	while (expected > 0) {
@@ -231,11 +235,27 @@ camera_abilities (CameraAbilitiesList *list)
 	a.usb_product	= 0x2202;
 	gp_abilities_list_append(list, a);
 
+	strcpy(a.model, "Ricoh:Caplio 300G");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2203;
+	gp_abilities_list_append(list, a);
+
 	strcpy(a.model, "Ricoh:Caplio G4");
 	a.usb_vendor	= 0x5ca;
 	a.usb_product	= 0x2208;
 	gp_abilities_list_append(list, a);
 
+	strcpy(a.model, "Ricoh:Capilo RX");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x220b;
+	gp_abilities_list_append(list, a);
+
+
+	strcpy(a.model, "Medion:MD 6126");
+	a.usb_vendor    = 0x5ca;
+	a.usb_product   = 0x2205;
+	gp_abilities_list_append(list, a);
+
 	return (GP_OK);
 }
 
@@ -693,6 +713,9 @@ out:
 	return ret;
 }
 
+/* for DOS FAT -> UNIX time conversion */
+static int day_n[] = { 0,31,59,90,120,151,181,212,243,273,304,334,0,0,0,0 };
+
 static int
 file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		void *data, GPContext *context)
@@ -700,6 +723,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 	Camera *camera = data;
 	char *buf = NULL, *reply = NULL, *cmd;
 	int ret = GP_OK;
+	unsigned char *ubuf;
 
 	cmd = malloc(6 + strlen(folder) + 1);
 	strcpy(cmd, "-NLST ");
@@ -715,9 +739,12 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		if (ret < GP_OK) goto out;
 		gp_log(GP_LOG_DEBUG, "g3" , "reply %s", reply);
 
+		ubuf = (unsigned char*)buf;
 		for (n=0;n < len/32;n++) {
 			if (buf[n*32+11] == 0x20) {
+				CameraFileInfo  info;
 				char xfn[13];
+				int date, time, month, year;
 
 				strcpy(xfn, buf+n*32);
 				xfn[8] = '.';
@@ -726,6 +753,52 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 
 				ret = gp_filesystem_append (fs, folder, xfn, context);
 				if (ret < GP_OK) goto out;
+
+				/* we also get parts of fs info for free, so just set it */
+				info.file.fields =
+						GP_FILE_INFO_NAME |
+						GP_FILE_INFO_SIZE |
+						GP_FILE_INFO_MTIME;
+				info.file.size =(ubuf[n*32+28]<<24)|
+						(ubuf[n*32+29]<<16)|
+						(ubuf[n*32+30]<< 8)|
+						(ubuf[n*32+31]    );
+				strcpy(info.file.name,xfn);
+				if (!strcmp(xfn+9,"JPG") || !strcmp(xfn+9,"jpg")) {
+					strcpy(info.file.type,GP_MIME_JPEG);
+					info.file.fields |= GP_FILE_INFO_TYPE;
+				}
+
+				if (!strcmp(xfn+9,"AVI") || !strcmp(xfn+9,"avi")) {
+					strcpy(info.file.type,GP_MIME_AVI);
+					info.file.fields |= GP_FILE_INFO_TYPE;
+				}
+
+				if (!strcmp(xfn+9,"WAV") || !strcmp(xfn+9,"wav")) {
+					strcpy(info.file.type,GP_MIME_WAV);
+					info.file.fields |= GP_FILE_INFO_TYPE;
+				}
+
+				if (!strcmp(xfn+9,"MTA") || !strcmp(xfn+9,"mta")) {
+					strcpy(info.file.type,"text/plain");
+					info.file.fields |= GP_FILE_INFO_TYPE;
+				}
+				info.preview.fields = 0;
+				date = (ubuf[n*32+16]) | (ubuf[n*32+17]<<8);
+				time = (ubuf[n*32+14]) | (ubuf[n*32+15]<<8);
+
+				/* from kernel fs/fat/, time_dos2unix. */
+        			month = ((date >> 5) - 1) & 15;
+				year = date >> 9;
+				info.file.mtime =
+					(time & 31)*2+60*((time >> 5) & 63)+
+					(time >> 11)*3600+86400*((date & 31)-1+
+					day_n[month]+(year/4)+year*365-
+					((year & 3) == 0 && month < 2 ? 1 : 0)+
+					3653);
+
+				ret = gp_filesystem_set_info_noop(fs, folder, info, context);
+ 
 			}
 		}
 	} else {
