@@ -1,8 +1,11 @@
 /* olympus_desc.c:
  *
- * Olympus-specific code: David Selmeczi <david@esr.elte.hu>
+ * Olympus 3040 specific code (most of this file):
+ * 	David Selmeczi <david@esr.elte.hu>
+ * Olympus 3000z code, based on data from:
+ * 	Till Kamppeter <till.kamppeter@gmx.net>
  * Original code:
- *  Copyright (C) 2002 Patrick Mansfield <patman@aracnet.com>
+ *  Copyright  2002 Patrick Mansfield <patman@aracnet.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,8 +23,9 @@
  * Boston, MA 02111-1307, USA.
  */
 #include <stdio.h>
+#include <_stdint.h>
 #include <sys/types.h>
-#include <malloc.h>
+#include <stdlib.h>
 #include <string.h>
 #include <gphoto2-library.h>
 #include "sierra.h"
@@ -29,7 +33,7 @@
 #include "sierra-desc.h"
 
 /*
- * Camera descriptor for Olympus C-3040Z (and possibly others).
+ * Camera descriptor for Olympus C-3040Z, 3000Z (and somday maybe others).
  */
 
 /*
@@ -143,7 +147,7 @@ static const RegisterDescriptorType oly3040_reg_03[] = {
 };
 #else
 static const ValueNameType oly3040_reg_03_val_names[] = { 
-	{ {       0 }, "Auto" },
+	{ {       0 }, N_("Auto") },
 	{ {    1250 }, "1/800" },
 	{ {    2000 }, "1/500" },
 	{ {    2500 }, "1/400" },
@@ -183,11 +187,11 @@ static const RegisterDescriptorType oly3040_reg_03[] = {
 #endif
 
 /*
- * Register 5: aperature settings. Works only in A or M mode
+ * Olympus 3040 Register 5: aperature settings. Works only in A or M mode
  * In A mode, shutter speed is compensated
  */
 static const ValueNameType oly3040_reg_05_val_names[] = {
-	{ { 0 }, "Auto" },
+	{ { 0 }, N_("Auto") },
 	{ { 1 }, "F1.8" },
 	{ { 2 }, "F2.0" },
 	{ { 3 }, "F2.3" },
@@ -213,15 +217,47 @@ static const RegisterDescriptorType oly3040_reg_05[] = {
 	}
 };
 
+/*
+ * Olympus 3000z Register 5: aperature settings. Unknown if this comment
+ * applies: Works only in A or M mode In A mode, shutter speed is
+ * compensated
+ */
+static const ValueNameType oly3000z_reg_05_val_names[] = {
+	{ { 0 }, N_("Auto") },
+	{ { 1 }, "F2.2" },
+	{ { 2 }, "F2.5" },
+	{ { 3 }, "F2.8" },
+	{ { 4 }, "F3.2" },
+	{ { 5 }, "F3.6" },
+	{ { 6 }, "F4.0" },
+	{ { 7 }, "F4.5" },
+	{ { 8 }, "F5.0" },
+	{ { 9 }, "F5.6" },
+	{ { 10 }, "F6.3" },
+	{ { 11 }, "F7.0" },
+	{ { 12 }, "F8.0" },
+	{ { 13 }, "F9.0" },
+	{ { 14 }, "F10.0" },
+	{ { 15 }, "F11.0" },
+	{ { 16 }, "F2.0" }, /* Odd */
+};
+static const RegisterDescriptorType oly3000z_reg_05[] = { 
+	{
+		GP_WIDGET_MENU, GP_REG_NO_MASK, 
+		"aperature", N_("Aperature Settings"),
+		VAL_NAME_INIT (oly3000z_reg_05_val_names)
+	}
+};
+
 /*
  * Register 6: color mode
  */
 static const ValueNameType oly3040_reg_06_val_names[] = {
-	{ { 0 }, "Normal" },
-	{ { 1 }, "B/W" },
-	{ { 2 }, "Sepia" },
-	{ { 3 }, "White board" },
-	{ { 4 }, "Black board" },
+	{ { 0 }, N_("Normal") },
+	{ { 1 }, N_("B/W") },
+	{ { 2 }, N_("Sepia") },
+	{ { 3 }, N_("White board") },
+	{ { 4 }, N_("Black board") },
 };
 static const RegisterDescriptorType oly3040_reg_06[] = { 
 	{
@@ -235,10 +271,10 @@ static const RegisterDescriptorType oly3040_reg_06[] = {
  * Register 7: flash settings
  */
 static const ValueNameType oly3040_reg_07_val_names[] = {
-	{ { 0 }, "Auto" },
-	{ { 1 }, "Force" },
-	{ { 2 }, "Off" },
-	{ { 3 }, "Anti-redeye" },
+	{ { 0 }, N_("Auto") },
+	{ { 1 }, N_("Force") },
+	{ { 2 }, N_("Off") },
+	{ { 3 }, N_("Anti-redeye") },
 };
 static const RegisterDescriptorType oly3040_reg_07[] = { 
 	{
@@ -249,14 +285,14 @@ static const RegisterDescriptorType oly3040_reg_07[] = {
 };
 
 /*
- * Register 20: white balance.
+ * Olympus 3040: Register 20: white balance.
  */
 static const ValueNameType oly3040_reg_20_val_names[] = {
-	{ { 0x00 }, "Auto" },
-	{ { 0x01 }, "Daylight" },
-	{ { 0x02 }, "Fluorescent" },
-	{ { 0x03 }, "Tungsten" },
-	{ { 0xff }, "Cloudy" },
+	{ { 0x00 }, N_("Auto") },
+	{ { 0x01 }, N_("Daylight") },
+	{ { 0x02 }, N_("Fluorescent") },
+	{ { 0x03 }, N_("Tungsten") },
+	{ { 0xff }, N_("Cloudy") },
 };
 static const RegisterDescriptorType oly3040_reg_20[] = { 
 	{
@@ -266,14 +302,32 @@ static const RegisterDescriptorType oly3040_reg_20[] = {
 	}
 };
 
+/*
+ * Olympus 3000z: Register 20: white balance. Only difference seems to be
+ * that cloudy is 0x04, not 0xff.
+ */
+static const ValueNameType oly3000z_reg_20_val_names[] = {
+	{ { 0x00 }, N_("Auto") },
+	{ { 0x01 }, N_("Daylight") },
+	{ { 0x02 }, N_("Fluorescent") },
+	{ { 0x03 }, N_("Tungsten") },
+	{ { 0x04 }, N_("Cloudy") },
+};
+static const RegisterDescriptorType oly3000z_reg_20[] = { 
+	{
+		GP_WIDGET_RADIO, GP_REG_NO_MASK,
+		"whitebalance", N_("White Balance"),
+		VAL_NAME_INIT (oly3000z_reg_20_val_names)
+	}
+};
 
 /*
  * Register 33: focus mode.
  */
 static const ValueNameType oly3040_reg_33_val_names[] = {
-	{ { 0x01 }, "Macro" },
-	{ { 0x02 }, "Auto" },
-	{ { 0x03 }, "Manual" },
+	{ { 0x01 }, N_("Macro") },
+	{ { 0x02 }, N_("Auto") },
+	{ { 0x03 }, N_("Manual") },
 };
 static const RegisterDescriptorType oly3040_reg_33[] = { 
 	{
@@ -292,9 +346,9 @@ static const RegisterDescriptorType oly3040_reg_33[] = {
  */
 
 static const ValueNameType oly3040_reg_34_val_names[] = {
-	{ { 0x01 }, "Off" },
-	{ { 0x02 }, "Monitor" },
-	{ { 0x03 }, "Normal" },
+	{ { 0x01 }, N_("Off") },
+	{ { 0x02 }, N_("Monitor") },
+	{ { 0x03 }, N_("Normal") },
 };
 static const RegisterDescriptorType oly3040_reg_34[] = { 
 	{
@@ -366,9 +420,9 @@ static const RegisterDescriptorType oly3040_reg_38[] = {
  */
 /*
 static const ValueNameType oly3040_reg_53_val_names[] = {
-	{ { 0x03 }, "English" },
-	{ { 0x04 }, "French" },
-	{ { 0x05 }, "German" },
+	{ { 0x03 }, N_("English") },
+	{ { 0x04 }, N_("French") },
+	{ { 0x05 }, N_("German") },
 };
 static const RegisterDescriptorType oly3040_reg_53[] = { 
 	{
@@ -409,8 +463,8 @@ static const RegisterDescriptorType oly3040_reg_69[] = {
  * Register 70: exposure meter.
  */
 static const ValueNameType oly3040_reg_70_val_names[] = {
-	{ { 0x03 }, "Spot" },
-	{ { 0x05 }, "Matrix" },
+	{ { 0x03 }, N_("Spot") },
+	{ { 0x05 }, N_("Matrix") },
 };
 static const RegisterDescriptorType oly3040_reg_70[] = { 
 	{
@@ -421,7 +475,23 @@ static const RegisterDescriptorType oly3040_reg_70[] = {
 };
 
 /*
- * Register 71: optical zoom value.
+ * Oly 750 uz Register 71: optical zoom value.
+ */
+static const ValueNameType oly750uz_reg_71_val_names[] = {
+	{
+		{ range: { 6.3, 63.0, .1 } }, NULL
+	}
+};
+static const RegisterDescriptorType oly750uz_reg_71[] = { 
+	{
+		GP_WIDGET_RANGE, GP_REG_NO_MASK, 
+		"zoom", N_("Zoom (in millimeters)"),
+		VAL_NAME_INIT (oly750uz_reg_71_val_names)
+	}
+};
+
+/*
+ * Oly 3040 Register 71: optical zoom value.
  */
 static const ValueNameType oly3040_reg_71_val_names[] = {
 	{
@@ -440,7 +510,7 @@ static const RegisterDescriptorType oly3040_reg_71[] = {
  * Register 72: digital zoom
  */
 static const ValueNameType oly3040_reg_72_val_names[] = {
-	{ { 0x0000 }, "Off" },
+	{ { 0x0000 }, N_("Off") },
 	{ { 0x0208 }, "1.5x" },
 	{ { 0x0008 }, "2x" },
 	{ { 0x0408 }, "2.5x" },
@@ -457,7 +527,7 @@ static const RegisterDescriptorType oly3040_reg_72[] = {
  * Register 85: ISO speed, read only
  */
 static const ValueNameType oly3040_reg_85_val_names[] = {
-	{ { 0 }, "Auto" },
+	{ { 0 }, N_("Auto") },
 	{ { 1 }, "100" },
 	{ { 2 }, "200" },
 	{ { 3 }, "400" },
@@ -490,7 +560,7 @@ static const RegisterDescriptorType oly3040_reg_103[] = {
  * Register 41: time format, read only
  */
 static const ValueNameType oly3040_reg_41_val_names[] = {
-	{ { 0 }, "Off" },
+	{ { 0 }, N_("Off") },
 	{ { 1 }, "yymmdd" },
 	{ { 2 }, "mmddyy" },
 	{ { 3 }, "ddmmyy" },
@@ -505,9 +575,9 @@ static const RegisterDescriptorType oly3040_reg_41[] = {
 
 
 /*
- * All of the register used to modify picture settings. The register value
- * received from the camera is stored into this data area, so it cannot be
- * a const.
+ * Olympus 3040: All of the register used to modify picture settings. The
+ * register value received from the camera is stored into this data area,
+ * so it cannot be a const.
  */
 static CameraRegisterType oly3040_pic_regs[] =  {
 	/* camera prefix, register number, size of register */
@@ -526,6 +596,27 @@ static CameraRegisterType oly3040_pic_regs[] =  {
 	CAM_REG_TYPE_INIT (oly3040, 85, 4, CAM_DESC_DEFAULT, 0), /* ISO Speed, read only  */
 };
 
+/*
+ * Olympus 3000z: All of the register used to modify picture settings.
+ * Note that most of these are the same as the 3040.
+ */
+static CameraRegisterType oly3000z_pic_regs[] =  {
+	/* camera prefix, register number, size of register */
+	CAM_REG_TYPE_INIT (oly3040, 01, 4, CAM_DESC_DEFAULT, 0), /* resolution/size */
+	CAM_REG_TYPE_INIT (oly3040, 03, 4, CAM_DESC_DEFAULT, 0), /* shutter */
+	CAM_REG_TYPE_INIT (oly3000z, 05, 4, CAM_DESC_DEFAULT, 0), /* aperature (f-stop) */
+	CAM_REG_TYPE_INIT (oly3040, 06, 4, CAM_DESC_DEFAULT, 0), /* color mode */
+	CAM_REG_TYPE_INIT (oly3040, 07, 4, CAM_DESC_DEFAULT, 0), /* flash */
+	CAM_REG_TYPE_INIT (oly3000z, 20, 4, CAM_DESC_DEFAULT, 0), /* white balance */
+	CAM_REG_TYPE_INIT (oly3040, 33, 4, CAM_DESC_DEFAULT, 0), /* focus mode */
+	CAM_REG_TYPE_INIT (oly3040,103, 4, CAM_DESC_DEFAULT, 0), /* focus position */
+	CAM_REG_TYPE_INIT (oly3040, 69, 8, CAM_DESC_DEFAULT, 0), /* exposure compensation */
+	CAM_REG_TYPE_INIT (oly3040, 70, 4, CAM_DESC_DEFAULT, 0), /* exposure metering */
+	CAM_REG_TYPE_INIT (oly3040, 71, 8, CAM_DESC_DEFAULT, 0), /* optical zoom */
+	CAM_REG_TYPE_INIT (oly3040, 72, 4, CAM_DESC_DEFAULT, 0), /* digital zoom + lense +  AE lock */
+	CAM_REG_TYPE_INIT (oly3040, 85, 4, CAM_DESC_DEFAULT, 0), /* ISO Speed, read only  */
+};
+
 /*
  * All of the register used to modify camera settings.
  */
@@ -551,6 +642,70 @@ static const CameraRegisterSetType oly3040_desc[] = {
 		},
 };
 
+/*
+ * Olympus 750UZ: All of the register used to modify picture settings.
+ *
+ * XXX patman for now all except zoom are the same as the 3000 - a mix of
+ * 3040 and 3000.
+ */
+static CameraRegisterType oly750uz_pic_regs[] =  {
+	/* camera prefix, register number, size of register */
+	CAM_REG_TYPE_INIT (oly3040, 01, 4, CAM_DESC_DEFAULT, 0), /* resolution/size */
+	CAM_REG_TYPE_INIT (oly3040, 03, 4, CAM_DESC_DEFAULT, 0), /* shutter */
+	CAM_REG_TYPE_INIT (oly3000z, 05, 4, CAM_DESC_DEFAULT, 0), /* aperature (f-stop) */
+	CAM_REG_TYPE_INIT (oly3040, 06, 4, CAM_DESC_DEFAULT, 0), /* color mode */
+	CAM_REG_TYPE_INIT (oly3040, 07, 4, CAM_DESC_DEFAULT, 0), /* flash */
+	CAM_REG_TYPE_INIT (oly3000z, 20, 4, CAM_DESC_DEFAULT, 0), /* white balance */
+	CAM_REG_TYPE_INIT (oly3040, 33, 4, CAM_DESC_DEFAULT, 0), /* focus mode */
+	CAM_REG_TYPE_INIT (oly3040,103, 4, CAM_DESC_DEFAULT, 0), /* focus position */
+	CAM_REG_TYPE_INIT (oly3040, 69, 8, CAM_DESC_DEFAULT, 0), /* exposure compensation */
+	CAM_REG_TYPE_INIT (oly3040, 70, 4, CAM_DESC_DEFAULT, 0), /* exposure metering */
+	CAM_REG_TYPE_INIT (oly750uz, 71, 8, CAM_DESC_DEFAULT, 0), /* optical zoom */
+	CAM_REG_TYPE_INIT (oly3040, 72, 4, CAM_DESC_DEFAULT, 0), /* digital zoom + lense +  AE lock */
+	CAM_REG_TYPE_INIT (oly3040, 85, 4, CAM_DESC_DEFAULT, 0), /* ISO Speed, read only  */
+};
+
+/*
+ * All of the register used to modify camera settings.
+ *
+ * XXX patman for now all the same as 3040.
+ */
+static const CameraRegisterType oly750uz_cam_regs[] = {
+	CAM_REG_TYPE_INIT (oly3040, 02, 4, CAM_DESC_DEFAULT, 0), /* date-time */
+	CAM_REG_TYPE_INIT (oly3040, 34, 4, CAM_DESC_SUBACTION, 
+			  SIERRA_ACTION_LCD_MODE), /* lcd mode */
+	CAM_REG_TYPE_INIT (oly3040, 35, 4, CAM_DESC_DEFAULT, 0), /* LCD brightness */
+	CAM_REG_TYPE_INIT (oly3040, 38, 4, CAM_DESC_DEFAULT, 0), /* LCD auto shutoff */
+	CAM_REG_TYPE_INIT (oly3040, 24, 4, CAM_DESC_DEFAULT, 0), /* Camera power save */
+	CAM_REG_TYPE_INIT (oly3040, 23, 4, CAM_DESC_DEFAULT, 0), /* Host power save */
+	CAM_REG_TYPE_INIT (oly3040, 41, 4, CAM_DESC_DEFAULT, 0), /* time format, read only */
+};
+
+static const CameraRegisterSetType oly750uz_desc[] = {
+		{ 
+			N_("Picture Settings"), 
+			SIZE_ADDR (CameraRegisterType, oly750uz_pic_regs)
+		},
+		{ 
+			N_("Camera Settings"), 
+			SIZE_ADDR (CameraRegisterType, oly750uz_cam_regs)
+		},
+};
+
+static const CameraRegisterSetType oly3000z_desc[] = {
+		{ 
+			N_("Picture Settings"), 
+			SIZE_ADDR (CameraRegisterType, oly3000z_pic_regs)
+		},
+		{ 
+			/*
+			 * Assumed that these are all the same as the 3040
+			 */
+			N_("Camera Settings"), 
+			SIZE_ADDR (CameraRegisterType, oly3040_cam_regs)
+		},
+};
+
 static const char oly3040_manual[] =
 N_(
 "Some notes about Olympus cameras:\n"
@@ -572,6 +727,24 @@ N_(
 "    PC and switch LCD to 'Off'."
 );
 
+static const char oly750uz_manual[] =
+N_(
+"Olympus 750 Ultra Zoom:\n"
+"(1) Olympus 750UZ has a USB PC Control mode. In order\n"
+"    to use this mode, the camera must be switched\n"
+"    into 'USB PC control mode'. To get to the menu\n"
+"    for switching modes, turn on the camera, open\n"
+"    the memory card access door and then press and\n"
+"    hold both the 'OK' and 'quickview' buttons until the\n"
+"    camera control menu appears. Set it to control mode.\n"
+"(2) If you switch the 'LCD mode' to 'Monitor' or\n"
+"    'Normal', don't forget to switch it back to 'Off'\n"
+"    before disconnectig. Otherwise you cannot use\n"
+"    the camera's buttons. If you end up with this\n"
+"    state, you should reconnect the camera to the\n"
+"    PC and switch LCD to 'Off'."
+);
+
 static const char default_manual[] = 
 N_(
 "Default sierra driver:\n\n"
@@ -584,9 +757,14 @@ N_(
 "    that the configuration settings are\n"
 "    complete for your camera, or can contribute\n"
 "    code to support complete configuration,\n"
-"    please contact gphoto-devel@gphoto.net\n"
+"    please contact the developer mailing list.\n"
 );
 
-const CameraDescType oly3040_cam_desc = { oly3040_desc, oly3040_manual, TRUE, };
+const CameraDescType oly750uz_cam_desc = { oly750uz_desc, oly750uz_manual,
+	SIERRA_EXT_PROTO, };
+const CameraDescType oly3040_cam_desc = { oly3040_desc, oly3040_manual,
+	SIERRA_EXT_PROTO, };
+const CameraDescType oly3000z_cam_desc = { oly3000z_desc, oly3040_manual,
+	SIERRA_EXT_PROTO, };
 const CameraDescType sierra_default_cam_desc = { oly3040_desc, default_manual,
-	FALSE, };
+	0, };
