@@ -1,6 +1,6 @@
 /* sierra_desc.h:
  *
- * Copyright (C) 2002 Patrick Mansfield <patman@aracnet.com>
+ * Copyright  2002 Patrick Mansfield <patman@aracnet.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -76,14 +76,14 @@ typedef struct {
 		 * range values.  The proper value to user here is based
 		 * on the reg_len size and the reg_widget_type.
 		 *
-		 * How do we handle things like the exp compensation, or
-		 * the zoom where the first x bytes are used, and it looks
-		 * like the rest are a fixed value? Let's just mask them
-		 * out, and use the XXX values read from the camera.
+		 * For fields like the exp compensation, or the zoom,
+		 * where the first x bytes are used, and it looks like the
+		 * rest are a fixed value: just mask them out, and use the
+		 * values read from the camera.
 		 */
 		int64_t value;	/* GP_WIDGET_RADIO */
-		float range[3];	/* GP_WIDGET_RANGE theser are
-				   min/max/stepping, if stepping is zero,
+		float range[3];	/* GP_WIDGET_RANGE these are
+				   min/max/stepping; if stepping is zero,
 				   a stepping of one is used. */
 	} u;
 	/* 
@@ -95,10 +95,10 @@ typedef struct {
 
 typedef struct {
 	CameraWidgetType reg_widget_type;
-	u_int32_t regs_mask;
+	uint32_t regs_mask;
 	char *regs_short_name;		/* for command line use */
 	char *regs_long_name;		/* for gui/curses use */
-	u_int32_t reg_val_name_cnt;	/* number of regs_value_names entries */
+	uint32_t reg_val_name_cnt;	/* number of regs_value_names entries */
 	ValueNameType *regs_value_names;	/* list of reg value/names */
 } RegisterDescriptorType;
 
@@ -107,33 +107,46 @@ typedef struct {
  * normally type cast its usage.
  */
 typedef struct {
-	u_int32_t reg_number; 	/* register number */
-	u_int32_t reg_len;	/* size in bytes of data at register */
-	u_int64_t reg_value; 	/* stored value, has no valid initial value;
+	uint32_t reg_number; 	/* register number */
+	uint32_t reg_len;	/* size in bytes of data at register */
+	uint64_t reg_value; 	/* stored value, has no valid initial value;
 				 * The size would have to be increased, or
 				 * this space must be allocated for
 				 * register values longer than 8 bytes. */
 	GetSetType	reg_get_set;
-	u_int32_t reg_desc_cnt;	/* number of reg_desc entries */
+	uint32_t reg_desc_cnt;	/* number of reg_desc entries */
 	RegisterDescriptorType *reg_desc; /* list of reg descriptors */
 } CameraRegisterType;
 
 typedef struct CameraRegisterSet {
 	char *window_name;
-	u_int32_t reg_cnt;
+	uint32_t reg_cnt;
 	CameraRegisterType *regs;
 } CameraRegisterSetType;
 
 typedef struct CameraDesc {
 	const CameraRegisterSetType *regset;
 	const char * const manual;
-	const unsigned use_extended_protocol:1;
+	const SierraFlags flags;
 } CameraDescType;
 
+/*
+ * oly-desc.c:
+ */
 extern const CameraDescType sierra_default_cam_desc;
+extern const CameraDescType oly750uz_cam_desc;
 extern const CameraDescType oly3040_cam_desc;
+extern const CameraDescType oly3000z_cam_desc;
+
+/*
+ * nikon-desc.c:
+ */
 extern const CameraDescType cp880_cam_desc;
 extern const CameraDescType cp995_cam_desc;
+
+/*
+ * epson-desc.c:
+ */
 extern const CameraDescType ep3000z_cam_desc;
 
 #endif /* __SIERRA_DESC_H__ */
