@@ -1,5 +1,5 @@
 /* Sony DSC-F55 & MSAC-SR1 - gPhoto2 camera library
- * Copyright  2001, 2002 Raymond Penners <raymond@dotsphinx.com>
+ * Copyright  2001, 2002, 2004 Raymond Penners <raymond@dotsphinx.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,26 +31,34 @@
 
 #define GP_MODULE "sonydscf55"
 
-int camera_id(CameraText * id)
+struct ModelInfo {
+	SonyModel model_id;
+	const char *model_str;
+};
+
+static struct ModelInfo models[] = {
+	{ SONY_MODEL_MSAC_SR1, "Sony:MSAC-SR1" },
+	{ SONY_MODEL_DCR_PC100, "Sony:DCR-PC100" },
+	{ SONY_MODEL_TRV_20E,   "Sony:TRV-20E" },
+	{ SONY_MODEL_DSC_F55, "Sony:DSC-F55" }
+};
+
+int
+camera_id(CameraText * id)
 {
 	strcpy(id->text, SONY_CAMERA_ID);
 	return (GP_OK);
 }
 
-int camera_abilities(CameraAbilitiesList * list)
+int
+camera_abilities(CameraAbilitiesList * list)
 {
-	static const char *models[] = {
-		SONY_MODEL_DSC_F55,
-		SONY_MODEL_MSAC_SR1,
-		SONY_MODEL_TRV_20E,
-		SONY_MODEL_DCR_PC100
-	};
 	int i;
 	CameraAbilities a;
 
 	for (i = 0; i < sizeof(models) / sizeof(models[i]); i++) {
 		memset(&a, 0, sizeof(a));
-		strcpy(a.model, models[i]);
+		strcpy(a.model, models[i].model_str);
 		a.status = GP_DRIVER_STATUS_PRODUCTION;
 		a.port = GP_PORT_SERIAL;
 		a.speed[0] = 0;
@@ -66,7 +74,8 @@ int camera_abilities(CameraAbilitiesList * list)
 /**
  * De-initialises camera
  */
-static int camera_exit(Camera * camera, GPContext *context)
+static int
+camera_exit(Camera * camera, GPContext *context)
 {
 	int rc;
 
@@ -86,7 +95,8 @@ static int camera_exit(Camera * camera, GPContext *context)
 
 
 
-static int camera_about(Camera * camera, CameraText * about, GPContext *context)
+static int
+camera_about(Camera * camera, CameraText * about, GPContext *context)
 {
 	strcpy(about->text,
 	       _("Sony DSC-F55/505 gPhoto library\n"
@@ -102,21 +112,72 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		void *data, GPContext *context)
 {
 	Camera *camera = data;
-	int count;
+	int mpeg, rc = GP_OK;
 
-	GP_DEBUG(
-			"camera_folder_list_files()");
+	GP_DEBUG("camera_folder_list_files()");
 
-	count = sony_image_count(camera);
-	if (count < 0)
-		return (count);
+	for (mpeg = 0; mpeg <= 1 && rc == GP_OK; mpeg++) {
+		int i, count;
+		SonyFileType file_type;
 
-	/* Populate the filesystem */
-	gp_list_populate(list, SONY_FILE_NAME_FMT, count);
+		file_type = mpeg ? SONY_FILE_MPEG : SONY_FILE_IMAGE;
 
+		rc = sony_file_count(camera, file_type, &count);
+		if (rc != GP_OK) {
+			break;
+		}
+
+		for (i = 1; i <= count; i++) {
+			char buf[13];
+			rc = sony_file_name_get(camera, i, file_type, buf);
+			if (rc != GP_OK) {
+				break;
+			}
+			gp_list_append(list, buf, NULL);
+
+			if (gp_context_cancel(context)
+			    == GP_CONTEXT_FEEDBACK_CANCEL) {
+				rc = GP_ERROR_CANCEL;
+			}
+		}
+	}
+	return rc;
+}
+
+
+static int
+get_sony_file_id(Camera *camera, const char *folder,
+		 const char *filename, GPContext *context,
+		 int *sony_id, SonyFileType *sony_type)
+{
+	int num = gp_filesystem_number(camera->fs, folder, filename, context);
+	if (num < 0)
+		return (num);
+
+	num++;
+
+	if (sony_is_mpeg_file_name(filename)) {
+		const char *name_found;
+		int mpeg_num = 0;
+		do {
+			mpeg_num++;
+			gp_filesystem_name(camera->fs, folder, num-mpeg_num, &name_found, context);
+		}
+		while (sony_is_mpeg_file_name(name_found)
+		       && (mpeg_num<=num));
+		mpeg_num--;
+
+		*sony_type = SONY_FILE_MPEG;
+		*sony_id = mpeg_num;
+	}
+	else {
+		*sony_type = SONY_FILE_IMAGE;
+		*sony_id = num;
+	}
 	return GP_OK;
 }
 
+
 static int
 get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	       CameraFileType type, CameraFile * file, void *data,
@@ -124,34 +185,46 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 {
 	Camera *camera = data;
 	int num;
+	SonyFileType file_type;
 	int rc = GP_ERROR;
 
-	GP_DEBUG(
-			"camera_file_get(\"%s/%s\")", folder, filename);
+	GP_DEBUG("camera_file_get(\"%s/%s\")", folder, filename);
 
-	num = gp_filesystem_number(camera->fs, folder, filename, context);
-	if (num < 0)
-		return (num);
-
-	num++;
-	GP_DEBUG(
-			"file %s has id %d", filename, num);
+	rc = get_sony_file_id(camera, folder, filename, context,
+			      &num, &file_type);
+	if (rc != GP_OK)
+		return rc;
 
 	switch (type) {
 	case GP_FILE_TYPE_NORMAL:
-		rc = sony_image_get(camera, num, file, context);
+		if (file_type == SONY_FILE_MPEG) {
+			rc = sony_mpeg_get(camera, num, file, context);
+                }
+		else {
+			rc = sony_image_get(camera, num, file, context);
+		}
 		break;
 	case GP_FILE_TYPE_PREVIEW:
-		rc = sony_thumbnail_get(camera, num, file, context);
+		if (file_type == SONY_FILE_MPEG) {
+			rc = GP_OK;
+		}
+		else {
+			rc = sony_thumbnail_get(camera, num, file, context);
+		}
 		break;
 	case GP_FILE_TYPE_EXIF:
-		rc = sony_exif_get(camera, num, file, context);
+		if (file_type == SONY_FILE_MPEG) {
+			rc = GP_OK;
+		}
+		else {
+			rc = sony_exif_get(camera, num, file, context);
+		}
 		break;
 	default:
 		rc = GP_ERROR_NOT_SUPPORTED;
 	}
 
-	
+
 	if (rc == GP_OK) {
 		gp_file_set_name (file, filename);
 	}
@@ -167,39 +240,103 @@ get_info_func (CameraFilesystem *fs, const char *folder,
 	       GPContext *context)
 {
 	Camera *camera = data;
-	int num;
+	int num, rc;
+	SonyFileType file_type;
 
-	num = gp_filesystem_number(camera->fs, folder, filename, context);
-	if (num < 0)
-		return (num);
+	rc = get_sony_file_id(camera, folder, filename, context,
+			      &num, &file_type);
+	if (rc != GP_OK)
+		return rc;
 
-	num++;
-	return (sony_image_info(camera, num, info, context));
+	rc = sony_image_info(camera, num, file_type, info, context);
+	return rc;
 }
 
+
+static int
+model_compare(const char *a, const char *b)
+{
+	const char *amod;
+	const char *bmod;
+	int alen, blen;
+	int rc;
+	
+	alen = strlen(a);
+	blen = strlen(b);
+	if (alen != blen) {
+		rc = 0;
+	}
+	else {
+		amod = strchr(a, ':');
+		bmod = strchr(b, ':');
+		if ((amod == NULL && bmod == NULL)
+		    || (amod != NULL && bmod != NULL)) {
+			rc = !strcasecmp(a, b);
+		}
+		else if (amod != NULL) {
+			int aidx = amod - a;
+			rc = (!strncasecmp(a, b, aidx))
+				&& (!strcasecmp(a+aidx+1, b+aidx+1));
+		}
+		else if (bmod != NULL) {
+			int bidx = bmod - b;
+			rc = (!strncasecmp(a, b, bidx))
+				&& (!strcasecmp(a+bidx+1, b+bidx+1));
+		}
+	}
+	return rc;
+}
+
+static int
+get_camera_model(Camera *camera, SonyModel *model)
+{
+	CameraAbilities a;
+	int rc;
+	
+	rc = gp_camera_get_abilities (camera, &a);
+	if (rc == GP_OK) {
+		int i;
+		rc = GP_ERROR;
+		for (i = 0; i < sizeof(models) / sizeof(models[i]); i++) {
+			if (model_compare(models[i].model_str,
+						 a.model)) {
+				rc = GP_OK;
+				*model = models[i].model_id;
+				break;
+			}
+		}
+	}
+	return rc;
+}
+
+
 /**
  * Initialises camera
  */
-int camera_init(Camera * camera, GPContext *context)
+int
+camera_init(Camera * camera, GPContext *context)
 {
-	CameraAbilities a;
-	int is_msac, rc;
+	int rc;
+	SonyModel model; 
+
+	rc = get_camera_model(camera, &model);
+	if (rc != GP_OK) {
+		return rc;
+	}
 
 	camera->functions->exit = camera_exit;
 	camera->functions->about = camera_about;
 
-	gp_camera_get_abilities (camera, &a);
-	is_msac = strcmp (a.model, SONY_MODEL_DSC_F55);
-
 	gp_filesystem_set_info_funcs (camera->fs, get_info_func, NULL, camera);
 	gp_filesystem_set_list_funcs (camera->fs, file_list_func, NULL, camera);
 	gp_filesystem_set_file_funcs (camera->fs, get_file_func, NULL, camera);
 
 	camera->pl = malloc (sizeof (CameraPrivateLibrary));
-	if (!camera->pl)
+	if (!camera->pl) {
 		return (GP_ERROR_NO_MEMORY);
+	}
 
-	rc = sony_init (camera, is_msac);
+	rc = sony_init (camera, model);
 	if (rc < 0) {
 		free (camera->pl);
 		camera->pl = NULL;
