@@ -99,6 +99,9 @@ models[] =
 		BRIDGE_SPCA504, SPCA50X_SDRAM },
 	{"Aiptek:Pencam", 0x04fc, 0x504a,
 		BRIDGE_SPCA504, SPCA50X_SDRAM | SPCA50X_FLASH },
+	/* same ids, different cam *sigh* */
+	{"Aiptek:Pencam without flash", 0x04fc, 0x504a,
+		BRIDGE_SPCA504, SPCA50X_SDRAM },
 	{"Medion:MD 5319", 0x04fc, 0x504a,
 		BRIDGE_SPCA504, SPCA50X_SDRAM | SPCA50X_FLASH },
 	{"nisis:Quickpix Qp3", 0x04fc, 0x504a,
@@ -120,7 +123,9 @@ models[] =
 	{"D-Link:DSC 350+", 0x084d, 0x0003,
 		BRIDGE_SPCA500, SPCA50X_FLASH},
         {"Minton:S-Cam F5", 0x084d, 0x0003,
-	        BRIDGE_SPCA500, SPCA50X_FLASH},
+	        BRIDGE_SPCA500, SPCA50X_FLASH},	
+	{"PureDigital:Ritz Disposable", 0x04fc, 0xffff,
+		BRIDGE_SPCA504B_PD, SPCA50X_FLASH},
 	{NULL, 0, 0, 0, 0}
 };
 
@@ -160,6 +165,9 @@ camera_abilities (CameraAbilitiesList *list)
 			 || a.usb_product == 0xc520)
 				a.operations = GP_OPERATION_CAPTURE_IMAGE;
 		}
+		if (models[x].bridge == BRIDGE_SPCA504B_PD) {
+                            a.operations = GP_OPERATION_CAPTURE_IMAGE;
+                }
 		if (models[x].bridge == BRIDGE_SPCA500) {
 			/* TEST enable capture for the DSC-350 style cams */
 			if (a.usb_vendor == 0x084d) {
@@ -582,10 +590,18 @@ camera_init (Camera *camera, GPContext *context)
 	while (model) {
 		if (abilities.usb_vendor == models[x].usb_vendor
 		 && abilities.usb_product == models[x].usb_product) {
-			camera->pl->bridge = models[x].bridge;
-			camera->pl->storage_media_mask =
-				models[x].storage_media_mask;
-			break;
+			int same;
+			char *m = strdup( models[x].model );
+			char *p = strchr (m, ':' );
+			*p = ' ';
+			same = !strcmp (m, abilities.model);
+			free (m);
+			if (same) {
+				camera->pl->bridge = models[x].bridge;
+				camera->pl->storage_media_mask =
+					models[x].storage_media_mask;
+				break;
+			}
 		}
 		model = models[++x].model;
 	}
@@ -594,15 +610,17 @@ camera_init (Camera *camera, GPContext *context)
 	if (camera->pl->fw_rev > 1) {
 		CHECK (spca50x_detect_storage_type (camera->pl));
 	}
+   
 	if (cam_has_flash(camera->pl) || cam_has_card(camera->pl) ) {
-		if (camera->pl->bridge == BRIDGE_SPCA504)
+		if ((camera->pl->bridge == BRIDGE_SPCA504) ||
+		    (camera->pl->bridge == BRIDGE_SPCA504B_PD))
 			CHECK (spca50x_flash_init (camera->pl, context));
 	}
 
-
-	if (camera->pl->bridge == BRIDGE_SPCA504) {
-		if (abilities.usb_vendor != 0x04fc 
-       && abilities.usb_product != 0x504a )
+	if ((camera->pl->bridge == BRIDGE_SPCA504) ||
+	    (camera->pl->bridge == BRIDGE_SPCA504B_PD)) {
+//		if (abilities.usb_vendor != 0x04fc && abilities.usb_product != 0x504a )
+		if (!(abilities.usb_vendor == 0x04fc && abilities.usb_product == 0x504a ))
 			ret = spca50x_reset (camera->pl);
 	}
 
@@ -636,13 +654,11 @@ cam_has_sdram (CameraPrivateLibrary *pl)
 static int
 cam_has_flash (CameraPrivateLibrary *pl)
 {
-
 	return pl->storage_media_mask & SPCA50X_FLASH;
 }
 
 static int
 cam_has_card (CameraPrivateLibrary *pl)
 {
-
 	return pl->storage_media_mask & SPCA50X_CARD;
 }
