@@ -92,6 +92,8 @@ struct camera_to_usb {
       	{ "DigitalDream:l'esprit",   0x0553, 0x0202, 0 },
 	/* http://www.digitaldreamco.com/shop/laronde.htm, VGA */
       	{ "DigitalDream:la ronde",   0x0553, 0x0202, 0 },
+	/* https://sf.net/tracker/index.php?func=detail&aid=1000498&group_id=8874&atid=358874 */
+	{ "DigitalDream:l'espion XS", 0x1183, 0x0001, 0 },
 	
 
 	/* reported by jerry white */ 
@@ -113,6 +115,9 @@ struct camera_to_usb {
 	/* Added for the konica e-mini by Roland Marcus Rutschmann<Rutschmann@gmx.de>*/
 	{ "Konica:e-mini",              0x04c8, 0x0722, 0 },
 
+	{ "Che-ez!:Babe",               0x0553, 0x0202, 0 },
+	{ "Che-ez!:SPYZ",               0x0553, 0x0202, 0 },
+
 	/* According to Sonic Tsunami <mmcburn86@earthlink.net> */
 	/* http://www.timlex.com/cif.html */
 	{ "Timlex:CP075",		0x0553, 0x0202, 1 }
