@@ -1,9 +1,9 @@
 /* gphoto2-camera.c
  *
- * Copyright (C) 2000 Scott Fritzinger
+ * Copyright  2000 Scott Fritzinger
  *
  * Contributions:
- * 	2001-2002: Lutz Mller <urc8@rz.uni-karlsruhe.de>
+ * 	2001-2002: Lutz Mller <lutz@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "gphoto2-camera.h"
 
 #include <stdlib.h>
@@ -193,6 +193,17 @@
 	}								\
 }
 
+#define CRSL(c,res,ctx,list)						\
+{									\
+	int r = (res);							\
+									\
+	if (r < 0) {							\
+		CAMERA_UNUSED (c,ctx);                              	\
+		gp_list_free (list);					\
+		return (r);						\
+	}								\
+}
+
 #define CHECK_RESULT_OPEN_CLOSE(c,result,ctx)				\
 {									\
 	int r;								\
@@ -210,6 +221,8 @@
 
 #define CHECK_INIT(c,ctx)						\
 {									\
+	if ((c)->pc->used)						\
+		return (GP_ERROR_CAMERA_BUSY);				\
 	(c)->pc->used++;						\
 	if (!(c)->pc->lh)						\
 		CR((c), gp_camera_init (c, ctx), ctx);			\
@@ -627,7 +640,6 @@ gp_camera_free (Camera *camera)
 int
 gp_camera_init (Camera *camera, GPContext *context)
 {
-        CameraList list;
 	CameraAbilities a;
 	const char *model, *port;
 	CameraLibraryInitFunc init_func;
@@ -636,7 +648,6 @@ gp_camera_init (Camera *camera, GPContext *context)
 	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Initializing camera...");
 
 	CHECK_NULL (camera);
-
 	/*
 	 * Reset the exit_requested flag. If this flag is set, 
 	 * gp_camera_exit will be called as soon as the camera is no
@@ -654,6 +665,11 @@ gp_camera_init (Camera *camera, GPContext *context)
 		GPPortInfoList *il;
 		int m, p;
 		GPPortInfo info;
+        	CameraList *list;
+
+		result = gp_list_new (&list);
+		if (result < GP_OK)
+			return result;
 
 		gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Neither "
 			"port nor model set. Trying auto-detection...");
@@ -663,25 +679,27 @@ gp_camera_init (Camera *camera, GPContext *context)
 		gp_abilities_list_load (al, context);
 		gp_port_info_list_new (&il);
 		gp_port_info_list_load (il);
-		gp_abilities_list_detect (al, il, &list, context);
-		if (!gp_list_count (&list)) {
+		gp_abilities_list_detect (al, il, list, context);
+		if (!gp_list_count (list)) {
 			gp_abilities_list_free (al);
 			gp_port_info_list_free (il);
 			gp_context_error (context, _("Could not detect "
 					     "any camera"));
+			gp_list_free (list);
 			return (GP_ERROR_MODEL_NOT_FOUND);
 		}
 
-		gp_list_get_name  (&list, 0, &model);
+		gp_list_get_name  (list, 0, &model);
 		m = gp_abilities_list_lookup_model (al, model);
 		gp_abilities_list_get_abilities (al, m, &a);
 		gp_abilities_list_free (al);
-		CRS (camera, gp_camera_set_abilities (camera, a), context);
-		CRS (camera, gp_list_get_value (&list, 0, &port), context);
+		CRSL (camera, gp_camera_set_abilities (camera, a), context, list);
+		CRSL (camera, gp_list_get_value (list, 0, &port), context, list);
 		p = gp_port_info_list_lookup_path (il, port);
 		gp_port_info_list_get_info (il, p, &info);
 		gp_port_info_list_free (il);
-		CRS (camera, gp_camera_set_port_info (camera, info), context);
+		CRSL (camera, gp_camera_set_port_info (camera, info), context, list);
+		gp_list_free (list);
 	}
 
 	if (strcasecmp (camera->pc->a.model, "Directory Browse")) {
@@ -958,7 +976,7 @@ int
 gp_camera_capture (Camera *camera, CameraCaptureType type,
 		   CameraFilePath *path, GPContext *context)
 {
-	CHECK_NULL (camera && path);
+	CHECK_NULL (camera);
 	CHECK_INIT (camera, context);
 
 	if (!camera->functions->capture) {
