@@ -1751,7 +1751,7 @@ gp_filesystem_lru_clear (CameraFilesystem *fs)
 	GP_DEBUG ("Clearing fscache LRU list...");
 
 	if (fs->lru_first == NULL) {
-		GP_DEBUG ("fscache LRU list already empty", n);
+		GP_DEBUG ("fscache LRU list already empty");
 		return (GP_OK);
 	}
 
@@ -1920,7 +1920,7 @@ gp_get_free_memory (GPContext *context, unsigned *free)
 #elif HAVE_SYSCTL && (__FreeBSD__ || __NetBSD__ || __OpenBSD__ || __APPLE__ )
 
 	int mib[2] = { CTL_HW, HW_PHYSMEM };
-	int value;
+	unsigned long value;
 	size_t valuelen = sizeof(value);
 	*free=0;
 	if (sysctl(mib, 2 , &value, &valuelen, NULL, 0) == -1) {
@@ -2086,7 +2086,7 @@ gp_filesystem_lru_check (CameraFilesystem *fs)
 	GP_DEBUG ("Checking fscache LRU list integrity...");
 
 	if (fs->lru_first == NULL) {
-		GP_DEBUG ("fscache LRU list empty", n);
+		GP_DEBUG ("fscache LRU list empty");
 		return (GP_OK);
 	}
 
