@@ -1,6 +1,6 @@
 /* gphoto2-widget.c
  *
- * Copyright (C) 2000 Scott Fritzinger
+ * Copyright  2000 Scott Fritzinger
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include <config.h>
+#include "config.h"
 #include "gphoto2-widget.h"
 
 #include <stdlib.h>
