@@ -78,12 +78,66 @@ int
 gp_port_library_list (GPPortInfoList *list)
 {
 	GPPortInfo info;
+	struct usb_bus *bus;
+	struct usb_device *dev;
+	int nrofdevices = 0;
 
+	/* default port first */
 	info.type = GP_PORT_USB;
 	strcpy (info.name, "Universal Serial Bus");
         strcpy (info.path, "usb:");
 	CHECK (gp_port_info_list_append (list, info));
 
+	usb_init ();
+	usb_find_busses ();
+	usb_find_devices ();
+
+	bus = usb_get_busses();
+	/* Look and enumerate all USB ports. */
+	while (bus) {
+		for (dev = bus->devices; dev; dev = dev->next) {
+			/* Devices which are definitely not cameras. */
+			if (	(dev->descriptor.bDeviceClass == USB_CLASS_HUB)		||
+				(dev->descriptor.bDeviceClass == USB_CLASS_HID)		||
+				(dev->descriptor.bDeviceClass == USB_CLASS_PRINTER)	||
+				(dev->descriptor.bDeviceClass == USB_CLASS_COMM)
+			)
+				continue;
+			/* Note: We do not skip USB storage. Some devices can support both,
+			 * and the Ricoh erronously reports it.
+			 */ 
+			nrofdevices++;
+		}
+		bus = bus->next;
+	}
+
+	/* If we already added usb:, and have 0 or 1 devices we have nothing to do.
+	 * This should be the standard use case.
+	 */
+	if (nrofdevices <= 1) 
+		return (GP_OK);
+
+	/* Redo the same bus/device walk, but now add the ports with usb:x,y notation,
+	 * so we can address all USB devices.
+	 */
+	bus = usb_get_busses();
+	while (bus) {
+		for (dev = bus->devices; dev; dev = dev->next) {
+			/* Devices which are definitely not cameras. */
+			if (	(dev->descriptor.bDeviceClass == USB_CLASS_HUB)		||
+				(dev->descriptor.bDeviceClass == USB_CLASS_HID)		||
+				(dev->descriptor.bDeviceClass == USB_CLASS_PRINTER)	||
+				(dev->descriptor.bDeviceClass == USB_CLASS_COMM)
+			)
+				continue;
+			/* Note: We do not skip USB storage. Some devices can support both,
+			 * and the Ricoh erronously reports it.
+			 */ 
+			sprintf (info.path, "usb:%s,%s", bus->dirname, dev->filename);
+			CHECK (gp_port_info_list_append (list, info));
+		}
+		bus = bus->next;
+	}
 	return (GP_OK);
 }
 
@@ -118,6 +172,7 @@ static int
 gp_port_usb_open (GPPort *port)
 {
 	int ret;
+	char name[64];
 
 	if (!port || !port->pl->d)
 		return GP_ERROR_BAD_PARAMETERS;
@@ -131,15 +186,39 @@ gp_port_usb_open (GPPort *port)
 		gp_port_set_error (port, _("Could not open USB device (%m)."));
 		return GP_ERROR_IO;
 	}
+#if defined(LIBUSB_HAS_GET_DRIVER_NP) && defined(LIBUSB_HAS_DETACH_KERNEL_DRIVER_NP)
+	memset(name,0,sizeof(name));
+	ret = usb_get_driver_np (port->pl->dh, port->settings.usb.interface,
+		name, sizeof(name)
+	);
+	if (strstr(name,"usbfs")) {
+		/* other gphoto instance most likely */
+		gp_port_set_error (port, _("Camera is already in use."));
+		return GP_ERROR_IO_LOCK;
+	}
+	if (strstr(name,"storage")) {
+		/* other gphoto instance most likely */
+		gp_port_set_error (port, _("Camera is supported by USB Storage driver."));
+		return GP_ERROR_NOT_SUPPORTED;
+	}
+	if (ret >= 0) {
+		gp_log (GP_LOG_DEBUG,"libusb",_("Device has driver '%s' attached, detaching it now."), name);
+		ret = usb_detach_kernel_driver_np (port->pl->dh, port->settings.usb.interface);
+		if (ret < 0)
+			gp_port_set_error (port, _("Could not detach kernel driver '%s' of camera device."),name);
+	} else {
+		gp_port_set_error (port, _("Could not query kernel driver of device."));
+	}
+#endif
 
 	ret = usb_claim_interface (port->pl->dh,
 				   port->settings.usb.interface);
 	if (ret < 0) {
 		gp_port_set_error (port, _("Could not claim "
 			"interface %d (%m). Make sure no other program "
-			"or kernel module (e.g. dc2xx or stv680) is using the device "
+			"or kernel module (such as %s) is using the device "
 			"and you have read/write access to the device."),
-			port->settings.usb.interface);
+			port->settings.usb.interface, "sdc2xx, stv680, spca50x");
 		return GP_ERROR_IO_USB_CLAIM;
 	}
 
@@ -185,6 +264,9 @@ gp_port_usb_clear_halt_lib(GPPort *port, int ep)
 	case GP_PORT_USB_ENDPOINT_OUT :
 		ret=usb_clear_halt(port->pl->dh, port->settings.usb.outep);
 		break;
+	case GP_PORT_USB_ENDPOINT_INT :
+		ret=usb_clear_halt(port->pl->dh, port->settings.usb.intep);
+		break;
 	default:
 		gp_port_set_error (port, "gp_port_usb_clear_halt: "
 				   "bad EndPoint argument");
@@ -280,7 +362,14 @@ gp_port_usb_update (GPPort *port)
 {
 	int ret;
 
-	if (!port || !port->pl->dh)
+	if (!port)
+		return GP_ERROR_BAD_PARAMETERS;
+
+	/* The portname can also be changed with the device still fully closed. */
+	memcpy(&port->settings.usb.port, &port->settings_pending.usb.port,
+		sizeof(port->settings.usb.port));
+
+ 	if (!port->pl->dh)
 		return GP_ERROR_BAD_PARAMETERS;
 
 	memcpy(&port->settings.usb, &port->settings_pending.usb,
@@ -377,10 +466,27 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 {
 	struct usb_bus *bus;
 	struct usb_device *dev;
+	char *s;
+	char busname[64], devname[64];
 
 	if (!port)
 		return (GP_ERROR_BAD_PARAMETERS);
 
+	s = strchr (port->settings.usb.port,':');
+	busname[0] = devname[0] = '\0';
+	if (s && (s[1] != '\0')) { /* usb:%d,%d */
+		strncpy(busname,s+1,sizeof(busname));
+		busname[sizeof(busname)-1] = '\0';
+
+		s = strchr(busname,',');
+		if (s) {
+			strncpy(devname, s+1,sizeof(devname));
+			devname[sizeof(devname)-1] = '\0';
+			*s = '\0';
+		} else {
+			busname[0] = '\0';
+		}
+	}
 	/*
 	 * NULL idvendor is not valid.
 	 * NULL idproduct is ok.
@@ -394,10 +500,16 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 	}
 
 	for (bus = usb_busses; bus; bus = bus->next) {
+		if ((busname[0] != '\0') && strcmp(busname, bus->dirname))
+			continue;
+
 		for (dev = bus->devices; dev; dev = dev->next) {
+			if ((devname[0] != '\0') && strcmp(devname, dev->filename))
+				continue;
+
 			if ((dev->descriptor.idVendor == idvendor) &&
 			    (dev->descriptor.idProduct == idproduct)) {
-				int config, interface, altsetting;
+				int config = -1, interface = -1, altsetting = -1;
 
 				port->pl->d = dev;
 
@@ -503,10 +615,27 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 {
 	struct usb_bus *bus;
 	struct usb_device *dev;
+	char *s;
+	char busname[64], devname[64];
 
 	if (!port)
 		return (GP_ERROR_BAD_PARAMETERS);
 
+	busname[0] = devname[0] = '\0';
+	s = strchr (port->settings.usb.port,':');
+	if (s && (s[1] != '\0')) { /* usb:%d,%d */
+		strncpy(busname,s+1,sizeof(busname));
+		busname[sizeof(busname)-1] = '\0';
+
+		s = strchr(busname,',');
+		if (s) {
+			strncpy(devname, s+1,sizeof(devname));
+			devname[sizeof(devname)-1] = '\0';
+			*s = '\0';
+		} else {
+			busname[0] = '\0';
+		}
+	}
 	/*
 	 * NULL class is not valid.
 	 * NULL subclass and protocol is ok.
@@ -517,8 +646,14 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 		return GP_ERROR_BAD_PARAMETERS;
 
 	for (bus = usb_busses; bus; bus = bus->next) {
+		if ((busname[0] != '\0') && strcmp(busname, bus->dirname))
+			continue;
+
 		for (dev = bus->devices; dev; dev = dev->next) {
-			int ret, config, interface, altsetting;
+			int ret, config = -1, interface = -1, altsetting = -1;
+
+			if ((devname[0] != '\0') && strcmp(devname, dev->filename))
+				continue;
 
 			ret = gp_port_usb_match_device_by_class(dev, class, subclass, protocol, &config, &interface, &altsetting);
 			if (!ret)
@@ -587,4 +722,3 @@ gp_port_library_operations (void)
 
 	return (ops);
 }
-
